CheckBoxList = class();

local Constants =
{
	Border = 1,
	ScrollBarSize = 10,
	SortIcon =
	{
		Width = 15,
		Height = 15,
		LabelWidth = 20,
		DropdownArrow = "Simbo/Poetical/Icons/DropdownArrow.tga"
	},
	Colours =
	{
		NormalOutline = Turbine.UI.Color(1.0, 0.8, 0.65, 0.1),
		HoverOutline = Turbine.UI.Color(0.8, 0.65, 0.1),
		SelectedItem = Turbine.UI.Color(0.90, 0.25, 0.25, 0.25)
	}
}

function CheckBoxList:Constructor(parent)
	self.DisplayItem = Turbine.UI.Control();
	self.DisplayItem:SetParent(parent);
	
	self.DisplayItem.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			self.SelectionList:SetVisible(not self.SelectionList:IsVisible());
			self.SelectionListBox:Focus();
		end
	end
	
	self.DisplayItemBox = Turbine.UI.Label();
	self.DisplayItemBox:SetParent(self.DisplayItem);
	self.DisplayItemBox:SetMouseVisible(false);
	self.DisplayItemBox:SetPosition(Constants.Border, Constants.Border);
	self.DisplayItemBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	
	self.SelectIconLabel = Turbine.UI.Label();
	self.SelectIconLabel:SetParent(self.DisplayItem);
	self.SelectIconLabel:SetMouseVisible(false);

	self.SelectIcon = Turbine.UI.Control();
	self.SelectIcon:SetParent(self.SelectIconLabel);
	self.SelectIcon:SetMouseVisible(false);
	self.SelectIcon:SetSize(Constants.SortIcon.Width, Constants.SortIcon.Height);
	self.SelectIcon:SetBackground(Constants.SortIcon.DropdownArrow);
	
	self.SelectionList = Turbine.UI.Control();
	self.SelectionList:SetParent(parent);
	self.SelectionList:SetZOrder(parent:GetZOrder() + 100);
	self.SelectionList:SetVisible(false);
	self.SelectionList.MouseLeave = function(sender, args)
		self.SelectionListBox:Focus();
	end
	
	self.SelectionListBox = Turbine.UI.ListBox();
	self.SelectionListBox:SetParent(self.SelectionList);
	self.SelectionListBox:SetPosition(Constants.Border, Constants.Border);
	self.SelectionListBox.HasFocus = false;

	self.SelectionListBox.FocusLost = function(sender, args)
		if not self.SelectionListBox.HasFocus then
			self.SelectionList:SetVisible(false);
		end
	end
	
	self.LinesScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.LinesScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.LinesScrollBar:SetParent(self.SelectionListBox);
	self.SelectionListBox:SetVerticalScrollBar(self.LinesScrollBar);
	self.LinesScrollBar.MouseEnter = function(sender, args)
		self.SelectionListBox.HasFocus = true;
	end
	self.LinesScrollBar.MouseLeave = function(sender, args)
		self.SelectionListBox.HasFocus = false;
	end
end

function CheckBoxList:SetPosition(left, top)
	self.DisplayItem:SetPosition(left, top);
	self.SelectionList:SetPosition(self.DisplayItem:GetLeft(), self.DisplayItem:GetTop() + self.DisplayItem:GetHeight() + (Constants.Border * 2));	
end

function CheckBoxList:SetSize(width, height, dropdownHeight)
	self.DisplayItem:SetSize(width, height);
	self.DisplayItemBox:SetSize(self.DisplayItem:GetWidth() - (Constants.Border * 2) - Constants.SortIcon.LabelWidth, self.DisplayItem:GetHeight() - (Constants.Border * 2));

	self.SelectIconLabel:SetSize(Constants.SortIcon.LabelWidth, self.DisplayItem:GetHeight() - (Constants.Border * 2));
	self.SelectIconLabel:SetPosition(self.DisplayItemBox:GetLeft() + self.DisplayItemBox:GetWidth(), Constants.Border);
	self.SelectIcon:SetPosition((self.SelectIconLabel:GetWidth() - Constants.SortIcon.Width) / 2, (self.SelectIconLabel:GetHeight() - Constants.SortIcon.Height) / 2);

	self.SelectionList:SetSize(width, dropdownHeight);
	self.SelectionList:SetPosition(self.DisplayItem:GetLeft(), self.DisplayItem:GetTop() + self.DisplayItem:GetHeight() + (Constants.Border * 2));	
	self.SelectionListBox:SetSize(self.DisplayItem:GetWidth() - (Constants.Border * 2), dropdownHeight - (Constants.Border * 2));
	for i = 1, self.SelectionListBox:GetItemCount() do
		self.SelectionListBox:GetItem(i):SetWidth(self.SelectionListBox:GetWidth());
	end
	
	self.LinesScrollBar:SetSize(Constants.ScrollBarSize,  self.SelectionListBox:GetHeight());
	self.LinesScrollBar:SetPosition(self.SelectionListBox:GetWidth() - Constants.ScrollBarSize,  0);
end

function CheckBoxList:GetLeft()
	return self.DisplayItem:GetLeft();
end

function CheckBoxList:GetWidth()
	return self.DisplayItem:GetWidth();
end

function CheckBoxList:SetWidth(width)
	self:SetSize(width, self.DisplayItem:GetHeight(), self.SelectionList:GetHeight());
end

function CheckBoxList:SetBorderColor(colour)
	self.DisplayItem:SetBackColor(colour);
	self.SelectionList:SetBackColor(colour);
end

function CheckBoxList:SetBackColor(colour)
	self.DisplayItemBox:SetBackColor(colour);
	self.SelectIconLabel:SetBackColor(colour);	
	self.SelectionListBox:SetBackColor(colour);	
end

function CheckBoxList:SetForeColor(colour)
	self.DisplayItemBox:SetForeColor(colour);
end

function CheckBoxList:SetFont(font)
	self.DisplayItemBox:SetFont(font);
end
	
function CheckBoxList:AddItem(text, selected)
	local itemCount = self.SelectionListBox:GetItemCount();
	local item = nil;
	local insertIndex = itemCount;
	
	for index = 1, itemCount do
		local checkItem = self.SelectionListBox:GetItem(index);
		local compare = text:Compare(checkItem.Value);

		if compare == 0 then
			item = checkItem;
			item:SetChecked(selected);
			break;
		elseif text:Compare(checkItem.Value) < 0 then
			insertIndex = index;
			break;
		else
			insertIndex = index + 1;
		end
	end

	if item == nil then
		item = Turbine.UI.Lotro.CheckBox();
		item:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
		item:SetSize(self.SelectionListBox:GetWidth() - Constants.ScrollBarSize, self.DisplayItemBox:GetHeight());
		item:SetText(" " .. text);
		item.Value = text;
		item:SetChecked(selected);

		item.MouseEnter = function(sender, args)
			item:SetOutlineColor(Constants.Colours.HoverOutline);
			item:SetFontStyle(Turbine.UI.FontStyle.Outline);
			self.SelectionListBox.HasFocus = true;
		end
		item.MouseLeave = function(sender, args)
			item:SetOutlineColor(Constants.Colours.NormalOutline);
			item:SetFontStyle(Turbine.UI.FontStyle.None);
			self.SelectionListBox.HasFocus = false;
		end
		item.CheckedChanged = function(sender, args)
			self:BuildDisplayBox();
		end
		
		self.SelectionListBox:InsertItem(insertIndex, item);
	end
	
	self:BuildDisplayBox();
end

function CheckBoxList:ClearItems()
	self.SelectionListBox:ClearItems();
	self.DisplayItemBox:SetText("");
end

function CheckBoxList:BuildDisplayBox()
	local text = self:GetText();
	self.DisplayItemBox:SetText(text);
end

function CheckBoxList:GetTextItems(selectedOnly)
	local itemCount = self.SelectionListBox:GetItemCount();
	local selectedItems = {};
	local selectedIndex = 1;
	for index = 1, itemCount do
		local item = self.SelectionListBox:GetItem(index);
		if not selectedOnly or item:IsChecked() then
			--table.insert(selectedItems, item.Value);
			selectedItems[tostring(selectedIndex)] = item.Value;
			selectedIndex = selectedIndex + 1;
		end
	end
	
	return selectedItems;
end

function CheckBoxList:GetText()
	local items = self:GetTextItems(true);
	local displayItems = ToArray(items);
	local display = table.concat(displayItems, ", ");
	
	return display;
end