import "Simbo.Babble.WindowBasic";

ConfirmationWindow = class(WindowBasic);

local Constants = 
{
	Normal = 
	{
		Size = { Width = 640, Height = 170 },
		Padding = { Top = 40, Left = 20, Right = 20, Bottom = 50 },
		TextBox = { Spacing = 20 },
		Buttons =
		{
			Padding = { Bottom = 20 },
			Spacing = 10
		}
	},
	Small =
	{
		Size = { Width = 320, Height = 150 },
		Padding = { Top = 40, Left = 20, Right = 20, Bottom = 50 },
		TextBox = { Spacing = 20 },
		Buttons =
		{
			Padding = { Bottom = 20 },
			Spacing = 10
		}
	}
}

function ConfirmationWindow:Constructor(size)
	WindowBasic.Constructor(self);
	
	local constants = Constants[size or "Normal"];
	
	self:SetSize(Turbine.UI.Display.GetSize());
	self:SetMouseVisible(true);
	
	self.MessageWindow = Turbine.UI.Lotro.Window();
	self.MessageWindow:SetParent(self);
	self.MessageWindow:SetSize(constants.Size.Width, constants.Size.Height);
	self.MessageWindow:SetPosition((self:GetWidth() - constants.Size.Width) / 2,(self:GetHeight() - constants.Size.Height) / 2);
	self.MessageWindow:SetVisible(true);
	self.MessageWindow.Closed = function(sender, args)
		self:Close();
	end

	self.MessageLabel = Turbine.UI.Label();
	self.MessageLabel:SetParent(self.MessageWindow);
	self.MessageLabel:SetPosition(constants.Padding.Left, constants.Padding.Top);
	self.MessageLabel:SetSize(self.MessageWindow:GetWidth() - constants.Padding.Left - constants.Padding.Right, self.MessageWindow:GetHeight() - constants.Padding.Top - constants.Padding.Bottom);
	self.MessageLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.MessageLabel:SetFont(Turbine.UI.Lotro.Font.TrajanPro16);
	self.MessageLabel:SetForeColor(Globals.Colours.Confirmation);
	self.MessageLabel:SetMultiline(true);
	self.MessageLabel:SetText(text);
	
	self.InputBox = CreateTextBox(self.MessageWindow);
	self.InputBox:SetPosition(constants.Padding.Left, self.MessageWindow:GetHeight() - constants.Padding.Bottom - self.InputBox:GetHeight());
	self.InputBox:SetWidth(self.MessageWindow:GetWidth() - constants.Padding.Left - constants.Padding.Right);
	self.InputBox:SetMultiline(false);
	self.InputBox:SetVisible(false);
	
	local cancelButton = CreateButton(self.MessageWindow);
	cancelButton:SetPosition((self.MessageWindow:GetWidth() - constants.Buttons.Spacing) / 2 - Globals.Sizes.Button.Width, self.MessageWindow:GetHeight() - constants.Buttons.Padding.Bottom - Globals.Sizes.Button.Height);
	cancelButton:SetText(Strings["cancel"]);
	
	local okButton = CreateButton(self.MessageWindow);
	okButton:SetPosition((self.MessageWindow:GetWidth() + constants.Buttons.Spacing) / 2, self.MessageWindow:GetHeight() - constants.Buttons.Padding.Bottom - Globals.Sizes.Button.Height);
	okButton:SetText(Strings["ok"]);

	cancelButton.Click = function(sender,args)
		self:Close();
	end
	
	okButton.Click = function(sender,args)
		local result = self.ButtonClick(sender, { ["Button"] = "OK", ["Text"] = self.InputBox:GetText():Trim() });
		if result == nil or result then
			self:Close();
		end
	end

	self.ButtonClick = function() end;
end

function ConfirmationWindow:ShowWindow(title, text, showTextBox, isModal, textBoxText)
	if showTextBox == nil then
		showTextBox = false;
	end
	
	if isModal == nil then
		isModal = true;
	end

	self.MessageWindow:SetText(title);
	self.MessageLabel:SetText(text);
	self.InputBox:SetText(textBoxText or "");
	self.InputBox:SetVisible(showTextBox);

	if showTextBox then
		self.MessageLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter);
	else
		self.MessageLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	end
	
	self:SetMouseVisible(isModal);
	self:SetVisible(true);
	self.MessageWindow:SetVisible(true);

	self:Activate();
	self.MessageWindow:Activate();
	
	if showTextBox then
		self.InputBox:Focus();
	end
end

function ConfirmationWindow:Hide()
	self:Close();
end