import "Simbo.Poetical.ConfirmationWindow";

EditorWindow = class(WindowLotro);

local Constants =
{
	MinimumSize =
	{
		Width = 570,
		Height = 400
	},
	SettingsBoxes = 
	{	
		Padding = { Left = 15, Top = 55, Right = 15, Bottom = 50},
		Spacing = 8,
		LabelSize = UIOptions["wide_editor_label"] and 160 or 140,
		RadioButtonSize = UIOptions["wide_editor_radio_buttons"] and 160 or 130,
		RadioButtonSizeLong = UIOptions["wide_editor_radio_buttons"] and 240 or 150,
		DropdownSize = 150
	},
	Buttons =
	{
		Padding = { Right = 15, Bottom = 20 },
		Spacing = 10
	},
	ScrollBarSize = 10,
	NewLine = "\n";
}


function EditorWindow:Constructor()
	WindowLotro.Constructor(self);
	
	self.CancelPressed = false;
	self.SavePressed = false;
	
	self:SetText("");
	self:SetPosition(tonumber(Settings.EditorWindow.Position.Left) or 150, tonumber(Settings.EditorWindow.Position.Top) or 150);
	self:SetSize(tonumber(Settings.EditorWindow.Size.Width) or Constants.MinimumSize.Width, tonumber(Settings.EditorWindow.Size.Height) or Constants.MinimumSize.Height);
	if self:GetWidth() < Constants.MinimumSize.Width then
		self:SetWidth(Constants.MinimumSize.Width);
	end
	if self:GetHeight() < Constants.MinimumSize.Height then
		self:SetHeight(Constants.MinimumSize.Height);
	end
	
	self:SetOpacity(GetOpacityFromSettings(Settings.EditorWindow));
	
	self.RowHeight = Globals.Sizes.Label.Height + Constants.SettingsBoxes.Spacing;
	
	local top = Constants.SettingsBoxes.Padding.Top;
	local nextJoinedRow = Globals.Sizes.Label.Height / 2 + Constants.SettingsBoxes.Spacing;
	
	local nameLabel = CreateLabel(self);
	nameLabel:SetPosition(Constants.SettingsBoxes.Padding.Left, top);
	nameLabel:SetWidth(Constants.SettingsBoxes.LabelSize);
	nameLabel:SetText(Strings["poem_name"]);
	
	self.NameBox = CreateTextBox(self);
	self.NameBox:SetPosition(Constants.SettingsBoxes.Padding.Left + Constants.SettingsBoxes.LabelSize, top);

	top = top + self.RowHeight;
	
	local tagsLabel = CreateLabel(self);
	tagsLabel:SetPosition(Constants.SettingsBoxes.Padding.Left, top);
	tagsLabel:SetWidth(Constants.SettingsBoxes.LabelSize);
	tagsLabel:SetText(Strings["tags"]);
	
	self.TagsBox = CheckBoxList(self);
	self.TagsBox:SetPosition(Constants.SettingsBoxes.Padding.Left + Constants.SettingsBoxes.LabelSize, top);
	self.TagsBox:SetFont(Globals.Fonts.ListBox);
	self.TagsBox:SetSize(Globals.Sizes.TextBox.Width, Globals.Sizes.TextBox.Height, Constants.SettingsBoxes.DropdownSize);
	self.TagsBox:SetBorderColor(Globals.BorderColours.ListBox);
	self.TagsBox:SetForeColor(Globals.Colours.ListBox);
	self.TagsBox:SetBackColor(Globals.BackgroundColours.ListBox);
	
	self.AddTagsWindow = ConfirmationWindow();
	self.AddTagsWindow:SetZOrder(self:GetZOrder() + 10);
	self.AddTagsWindow.ButtonClick = function (sender,args)
		local tags = args.Text:Split(",");
		for _, tag in ipairs(tags) do
			self.TagsBox:AddItem(tag:Trim(), true);
		end
	end
	
	self.AddTagsButton = CreateButton(self, UIOptions["wide_add_tag_button"]);
	self.AddTagsButton:SetText(Strings["new_tag"]);
	self.AddTagsButton:SetPosition(self.TagsBox:GetLeft() + Globals.Sizes.TextBox.Width + Constants.Buttons.Spacing, top);
	self.AddTagsButton.MouseClick = function(sender, args)
		self.AddTagsWindow:ShowWindow(Strings["new_tag"], Strings["new_tag_message"], true, false);
	end
	
	top = top + self.RowHeight;
	
	local defaultCommandLabel = CreateLabel(self);
	defaultCommandLabel:SetPosition(Constants.SettingsBoxes.Padding.Left, top);
	defaultCommandLabel:SetWidth(Constants.SettingsBoxes.LabelSize);
	defaultCommandLabel:SetText(Strings["default_cmd"]);
	
	self.DefaultCommandLocalRadioButton = CreateRadioButton(self, Strings["use_command"], "DefaultCommand", defaultCommandLabel:GetLeft() + defaultCommandLabel:GetWidth(), top, Constants.SettingsBoxes.RadioButtonSize, Globals.Sizes.Label.Height);
	
	self.DefaultCommandBox = CreateWideTextBox(self);
	self.DefaultCommandBox:SetPosition(self.DefaultCommandLocalRadioButton:GetLeft() + self.DefaultCommandLocalRadioButton:GetWidth(), top);
		
	top = top + nextJoinedRow;
	
	self.DefaultCommandGlobalRadioButton = CreateRadioButton(self, Strings["use_global"], "DefaultCommand", defaultCommandLabel:GetLeft() + defaultCommandLabel:GetWidth(), top, Constants.SettingsBoxes.RadioButtonSizeLong, Globals.Sizes.Label.Height);
	
	self.DefaultCommandGlobalLabel = CreateLabel(self);
	self.DefaultCommandGlobalLabel:SetPosition(self.DefaultCommandGlobalRadioButton:GetLeft() + self.DefaultCommandGlobalRadioButton:GetWidth(), top);
	self.DefaultCommandGlobalLabel:SetForeColor(Globals.Colours.SecondaryLabel);
	
	top = top + self.RowHeight;
	
	local blankLineCommandLabel = CreateLabel(self);
	blankLineCommandLabel:SetPosition(Constants.SettingsBoxes.Padding.Left, top);
	blankLineCommandLabel:SetWidth(Constants.SettingsBoxes.LabelSize);
	blankLineCommandLabel:SetText(Strings["blankline_action"]);
	
	self.BlankLineCommandLocalRadioButton = CreateRadioButton(self, Strings["use_command"], "BlankLine", blankLineCommandLabel:GetLeft() + blankLineCommandLabel:GetWidth(), top, Constants.SettingsBoxes.RadioButtonSize, Globals.Sizes.Label.Height);
	
	self.BlankLineCommandBox = CreateTextBox(self);
	self.BlankLineCommandBox:SetPosition(self.BlankLineCommandLocalRadioButton:GetLeft() + self.BlankLineCommandLocalRadioButton:GetWidth(), top);
		
	top = top + nextJoinedRow;
	
	self.BlankLineCommandSkipRadioButton = CreateRadioButton(self, Strings["skip_line"], "BlankLine", blankLineCommandLabel:GetLeft() + blankLineCommandLabel:GetWidth(), top, Constants.SettingsBoxes.RadioButtonSizeLong, Globals.Sizes.Label.Height);
		
	top = top + nextJoinedRow;
	
	self.BlankLineCommandGlobalRadioButton = CreateRadioButton(self, Strings["use_global"], "BlankLine", blankLineCommandLabel:GetLeft() + blankLineCommandLabel:GetWidth(), top, Constants.SettingsBoxes.RadioButtonSizeLong, Globals.Sizes.Label.Height);
	
	self.BlankLineCommandGlobalLabel = CreateLabel(self);
	self.BlankLineCommandGlobalLabel:SetPosition(self.BlankLineCommandGlobalRadioButton:GetLeft() + self.BlankLineCommandGlobalRadioButton:GetWidth(), top);
	self.BlankLineCommandGlobalLabel:SetForeColor(Globals.Colours.SecondaryLabel);

	top = top + self.RowHeight;
	
	local findReplaceButton = ExpandCollapseButton(self);
	findReplaceButton:SetPosition(Constants.SettingsBoxes.Padding.Left, top - findReplaceButton:GetHeight()- 4);
	findReplaceButton:SetCollapsedState(true)
	findReplaceButton.Clicked = function(sender, args)
		self.FindReplacePanel:SetVisible(not args.Collapsed);
		self:LayoutPoemBox();
	end
	
	self.FindReplacePanel = Turbine.UI.Control();
	self.FindReplacePanel:SetParent(self);
	self.FindReplacePanel:SetPosition(Constants.SettingsBoxes.Padding.Left, top);
	self.FindReplacePanel:SetVisible(false);
	
	self.FindBox = CreateTextBox(self.FindReplacePanel);
	self.FindBox:SetPosition(0, 0);

	self.FindButton = CreateButton(self.FindReplacePanel);
	self.FindButton:SetPosition(self.FindBox:GetLeft() + self.FindBox:GetWidth() + Constants.SettingsBoxes.Spacing, 0);
	self.FindButton:SetText(Strings["find"]);
	self.FindButton.MouseEnter = function(sender, args)
		local cursorPos = self.PoemBox:GetSelectionStart();
		self.PoemBox.Selection = { CursorPostion = cursorPos, Length = self.PoemBox:GetSelectionLength() };
	end
	self.FindButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			local text = self.FindBox:GetText();
			if string.len(text) > 0 then
				self:FindText(text);
			else
				self.FindBox:Focus();
			end
		end
	end
	
	self.ReplaceBox = CreateTextBox(self.FindReplacePanel);
	self.ReplaceBox:SetPosition(self.FindButton:GetLeft() + self.FindButton:GetWidth() + Constants.SettingsBoxes.Spacing, 0);
	
	self.ReplaceButton = CreateButton(self.FindReplacePanel);
	self.ReplaceButton:SetPosition(self.ReplaceBox:GetLeft() + self.ReplaceBox:GetWidth() + Constants.SettingsBoxes.Spacing, 0);
	self.ReplaceButton:SetText(Strings["replace"]);
	self.ReplaceButton.MouseEnter = function(sender, args)
		local cursorPos = self.PoemBox:GetSelectionStart();
		self.PoemBox.Selection = { CursorPostion = cursorPos, Length = self.PoemBox:GetSelectionLength() };
	end
	self.ReplaceButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			local text = self.FindBox:GetText();
			if string.len(text) > 0 then
				if string.ToLower(self.PoemBox:GetSelectedText()) ~= string.ToLower(text) then
					self:FindText(text);
				else
					local replaceText = self.ReplaceBox:GetText();
					self.PoemBox:SetSelectedText(replaceText);
					self:FindText(text);
				end
			else
				self.FindBox:Focus();
			end
		end
	end
	
	self.ReplaceAllButton = CreateButton(self.FindReplacePanel, UIOptions["wide_replace_all_button"]);
	self.ReplaceAllButton:SetPosition(self.ReplaceButton:GetLeft() + self.ReplaceButton:GetWidth() + Constants.SettingsBoxes.Spacing, 0);
	self.ReplaceAllButton:SetText(Strings["replace_all"]);
	self.ReplaceAllButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			local text = string.ToLower(self.FindBox:GetText());
			if string.len(text) > 0 then
				local replaceText = self.ReplaceBox:GetText();
				local poemText = self.PoemBox:GetText();
				
				local doReplace = true;
				local findIndex = 1;
				local count = 0;
				while doReplace do
					local first, last = string.ToLower(poemText):find(text, findIndex, true);
					if first ~= nil and count < 1000 then
						poemText = poemText:sub(1, first - 1) .. replaceText .. poemText:sub(last + 1)
						findIndex = first + replaceText:len();
						count = count + 1;
					else
						doReplace = false;
					end
				end
				self.PoemBox:SetText(poemText);
			else
				self.FindBox:Focus();
			end
		end
	end	

	self.SelectAllButton = CreateButton(self.FindReplacePanel, UIOptions["wide_select_all_button"]);
	self.SelectAllButton:SetPosition(self.FindBox:GetLeft(), self.FindBox:GetTop() + self.RowHeight);
	self.SelectAllButton:SetText(Strings["select_all"]);
	self.SelectAllButton.MouseClick = function(sender, args)
		self.PoemBox:SelectAll();
		self.PoemBox:Focus();
	end

	self.PoemBox = CreateTextBox(self, true);
	self.PoemBox:SetSelectable(true);
	self.PoemBox:SetPosition(Constants.SettingsBoxes.Padding.Left, top);
	
	self.PoemBoxScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.PoemBoxScrollBar:SetParent(self.PoemBox);
	self.PoemBoxScrollBar:SetZOrder(self.PoemBox:GetZOrder() + 1);
	self.PoemBoxScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.PoemBoxScrollBar:SetEnabled(true);
	
	self.PoemBox:SetVerticalScrollBar(self.PoemBoxScrollBar);
	
	self.ApplyButton = CreateButton(self);
	self.ApplyButton:SetText(Strings["apply"]);
	self.ApplyButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			self:SavePoem(false);
		end
	end	
	
	self.SaveButton = CreateButton(self);
	self.SaveButton:SetText(Strings["save"]);
	self.SaveButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			self:SavePoem(true);
		end
	end	
	
	self.CancelButton = CreateButton(self);
	self.CancelButton:SetText(Strings["cancel"]);
	self.CancelButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			self.CancelPressed = true;
			self:Close();
		end
	end	
	
	self.CloseConfirmation = ConfirmationWindow();
	self.CloseConfirmation:SetZOrder(self:GetZOrder() + 10);
	self.CloseConfirmation.ButtonClick = function (sender,args)
		self.CancelPressed = true;
		self:Close();
	end
	
	self.Closing = function(sender, args)
		if not (self.CancelPressed or self.SavePressed) then
			args.Cancel = true;
			self.CloseConfirmation:ShowWindow(self:GetText(), Strings["exit_without_save"], false, true);
		else
			self.CancelPressed = false;
			self.SavePressed = false;
		end
	end
	
	self.VisibleChanged = function(sender, args)
		if not self:IsVisible() then
			self.AddTagsWindow:SetVisible(false);
		else
			CheckWindowPosition(self);
		end
	end
	
	self.Saved = function() end;
	
	self:LayoutControls();
	self:SetResizer(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
end

function EditorWindow:ValidatePoem()
	local valid = true;
	
	self.NameBox:SetText(self.NameBox:GetText():Trim());
	self.DefaultCommandBox:SetText(self.DefaultCommandBox:GetText():Trim());
	self.BlankLineCommandBox:SetText(self.BlankLineCommandBox:GetText():Trim());
	self.PoemBox:SetText(self.PoemBox:GetText():Trim());
	
	if self.NameBox:GetText() == "" then
		Turbine.Shell.WriteLine("<rgb=#FF0000>" .. Strings["poem_title_required"] .. "</rgb>");
		valid = false;
	elseif self.PoemBox:GetText() == "" then
		Turbine.Shell.WriteLine("<rgb=#FF0000>" .. Strings["poem_text_required"] .. "</rgb>");
		valid = false;
	end
	
	return valid;
end

function EditorWindow:ShowWindow(poem)
	if poem ~= nil then
		self:SetText(Strings["edit_poem_with_separator"] .. poem.Title);
	else
		poem = 
		{ 
			Title = "", 
			DefaultCommand = "", 
			UseLocalDefaultCommand = false, 
			BlankLineCommand = "", 
			UseLocalBlankLineCommand = false,
			SkipBlankLineCommand = false,
			Lines = {} 
		};
		self:SetText(Strings["new_poem"]);
	end

	self.NameBox:SetText(poem.Title);
	self.TagsBox:ClearItems();
	if PoemDB.Tags ~= nil then
		local index = 1;
		local done = false;
		while not done do
			local tag = PoemDB.Tags[index] or PoemDB.Tags[tostring(index)];
			if tag ~= nil then
				local selected = IsInList(poem.Tags or {}, tag);
				self.TagsBox:AddItem(tag, selected);
				index = index + 1;
			else
				done = true;
			end
		end
	end
	
	self.DefaultCommandBox:SetText(poem.DefaultCommand);
	if poem.UseLocalDefaultCommand then
		self.DefaultCommandLocalRadioButton:SetSelected();
	else
		self.DefaultCommandGlobalRadioButton:SetSelected();
	end
	
	if poem.UseLocalBlankLineCommand then
		self.BlankLineCommandLocalRadioButton:SetSelected();
	elseif poem.SkipBlankLineCommand then
		self.BlankLineCommandSkipRadioButton:SetSelected();
	else
		self.BlankLineCommandGlobalRadioButton:SetSelected();
	end
	
	self.BlankLineCommandBox:SetText(poem.BlankLineCommand);
	
	self:ShowDefaultCommands();
	
	self.FindBox:SetText("");
	self.ReplaceBox:SetText("");
	
	self.PoemBox:SetText(" " .. table.concat(ToArray(poem.Lines), Constants.NewLine));
	self.PoemBox:SetSelection(1, 1);
	self.PoemBox:SetSelectedText("");
	self.PoemBox:SetSelection(1, 0);

	self.CancelPressed = false;
	self.SavePressed = false;
	
	self:SetVisible(true);
	self:ActivateWindow();
end

function EditorWindow:ActivateWindow()
	self.AddTagsWindow:SetVisible(false);
	self:Activate();
end

function EditorWindow:ShowDefaultCommands()
	local defaultText = "";
	if Settings.DefaultCommand == nil or string.len(Settings.DefaultCommand) == 0 then
		defaultText = "[" .. Strings["no_prefix"] .. "]";
	else
		defaultText = Settings.DefaultCommand;
	end
	self.DefaultCommandGlobalLabel:SetText(" (" .. defaultText .. ")");
	
	defaultText = "";
	if Settings.UseLocalBlankLineCommand then
		if Settings.BlankLineCommand == nil or string.len(Settings.BlankLineCommand) == 0 then
			defaultText = "[" .. Strings["no_action"] .. "]";
		else
			defaultText = Settings.BlankLineCommand;
		end
	else
		defaultText = "[" .. Strings["skip_line"] .. "]";
	end
	self.BlankLineCommandGlobalLabel:SetText(" (" .. defaultText .. ")");
end

function EditorWindow:LayoutControls()
	local width = self:GetWidth() - Constants.SettingsBoxes.Padding.Left - Constants.SettingsBoxes.Padding.Right;
	local boxWidth = width - Constants.SettingsBoxes.LabelSize;
	
	self.NameBox:SetWidth(boxWidth);
	self.TagsBox:SetWidth(boxWidth - self.AddTagsButton:GetWidth() - Constants.Buttons.Spacing);
	self.AddTagsButton:SetLeft(self.TagsBox:GetLeft() + self.TagsBox:GetWidth() + Constants.Buttons.Spacing)
	self.BlankLineCommandBox:SetWidth(boxWidth - Constants.SettingsBoxes.RadioButtonSize);
	self.DefaultCommandGlobalLabel:SetWidth(boxWidth - Constants.SettingsBoxes.RadioButtonSizeLong);
	self.BlankLineCommandGlobalLabel:SetWidth(boxWidth - Constants.SettingsBoxes.RadioButtonSizeLong);
	self.FindReplacePanel:SetSize(width, self.RowHeight * 2);
	
	self:LayoutPoemBox();

	self.SaveButton:SetPosition(self:GetWidth() - Constants.Buttons.Padding.Right - Globals.Sizes.Button.Width, self:GetHeight() - Constants.Buttons.Padding.Bottom - Globals.Sizes.Button.Height);
	self.ApplyButton:SetPosition(self.SaveButton:GetLeft() - Constants.Buttons.Spacing - Globals.Sizes.Button.Width, self.SaveButton:GetTop());
	self.CancelButton:SetPosition(self.ApplyButton:GetLeft() - Constants.Buttons.Spacing - Globals.Sizes.Button.Width, self.SaveButton:GetTop());
end

function EditorWindow:LayoutPoemBox()
	local width = self:GetWidth() - Constants.SettingsBoxes.Padding.Left - Constants.SettingsBoxes.Padding.Right;
	local ajdustHeight = 0;
	if self.FindReplacePanel:IsVisible() then
		ajdustHeight = self.RowHeight * 2;
	end
	
	self.PoemBox:SetTop(self.FindReplacePanel:GetTop() + ajdustHeight);
	self.PoemBox:SetSize(width, self:GetHeight() - self.PoemBox:GetTop() - Constants.SettingsBoxes.Padding.Bottom);

	self.PoemBoxScrollBar:SetPosition(self.PoemBox:GetWidth() - Constants.ScrollBarSize, 1);
	self.PoemBoxScrollBar:SetSize(Constants.ScrollBarSize, self.PoemBox:GetHeight());
end

function EditorWindow:FindText(text)
	local poemText = string.ToLower(self.PoemBox:GetText());
	local textToFind = string.ToLower(text);
	
	local cursorPos = 1;
	if self.PoemBox.Selection ~= nil then
		cursorPos = self.PoemBox.Selection.CursorPostion;

		if self.PoemBox.Selection.Length > 0 then
			cursorPos = cursorPos + 1;
		end
		
		if cursorPos > poemText:CharLen() then
			cursorPos = 1;
		end
	end
	
	local bytesCursorPos = poemText:GetByteCount(cursorPos);
	local pos = string.find(poemText, textToFind, bytesCursorPos, true);
	if pos == nil then
		pos = string.find(poemText, textToFind, 1, true);
	end
	
	if pos ~= nil then
		local adjustPos = string.sub(poemText, 1, pos - 1):CharLen() + 1;
		local adjustLength = textToFind:CharLen();
		
		self.PoemBox:SetSelection(adjustPos, adjustLength);
		self.PoemBox.Selection.CursorPostion = adjustPos;
		self.PoemBox.Selection.Length = adjustLength;
	end
end

function EditorWindow:SavePoem(closeWindow)
	if self:ValidatePoem() then
		local args = 
		{ 
			Poem = 
			{
				Title = self.NameBox:GetText(),
				Tags = self.TagsBox:GetTextItems(true),
				DefaultCommand = self.DefaultCommandBox:GetText(),
				UseLocalDefaultCommand = self.DefaultCommandLocalRadioButton:IsSelected(),
				BlankLineCommand = self.BlankLineCommandBox:GetText(),
				UseLocalBlankLineCommand = self.BlankLineCommandLocalRadioButton:IsSelected(),
				SkipBlankLineCommand = self.BlankLineCommandSkipRadioButton:IsSelected(),
				Lines = self.PoemBox:GetText():SplitLines();
			}
		};
		self.Saved(sender, args);
		
		if closeWindow then
			self.SavePressed = true;
			self:Close();
		end
	end
end