function string:ToLower()
	if self == nil or string.len(self) == 0 then
		return self;
	end
	
	local byteIndex = 1;
	local lowerCase = {};
	
	while byteIndex <= #self do
		local byteValue = string.byte(self, byteIndex);
		if byteValue >= 192 then
			local checkString =  string.sub(self, byteIndex, byteIndex + 1);
			local map = LowerCaseMap[checkString];
			if map ~= nil then
				table.insert(lowerCase, map);
			else
				table.insert(lowerCase, checkString);
			end
			byteIndex = byteIndex + 1;
		else
			table.insert(lowerCase, string.lower(string.char(byteValue))); 
		end
		
		byteIndex = byteIndex + 1;
	end
	
	return table.concat(lowerCase);
end

function string:Trim()
	local from = self:match"^%s*()";
	return from > #self and "" or self:match(".*%S", from);
end

function string:Split(separator)
	local fields = {}
	
	separator = separator or ":";
	local pattern = string.format("([^%s]+)", separator);
	
	self:gsub(pattern, function(c) 
		if c:Trim() ~= "" then
			fields[#fields+1] = c:Trim() 
		end
	end);
	
	return fields;
end

function string:SplitLines()
	local splitLines = {};
	local trimmed = self:Trim();
	local index = 1;
	
	local function helper(line) 
		--table.insert(splitLines, line:Trim()) 
		splitLines[tostring(index)] = line:Trim();
		index = index + 1;
		return "" 
	end
	helper((trimmed:gsub("(.-)\r?\n", helper)));
	
	return splitLines;
end

function string:GetByteCount(charCount)
	local charsIndex = 1;
	local byteCount = 0;
	
	while charsIndex <= charCount and byteCount < #self do
		if string.byte(self, byteCount + 1) >= 192 then
			byteCount = byteCount + 2;
		else
			byteCount = byteCount + 1;
		end
		charsIndex = charsIndex + 1;
	end
	return byteCount;
end

function string:CharLen()
	local chars = 0;
	local byteIndex = 1;
	
	while byteIndex <= #self do
		chars = chars + 1
		if string.byte(self, byteIndex) >= 192 then
			byteIndex = byteIndex + 2;
		else
			byteIndex = byteIndex + 1;
		end
	end
	return chars;
end

function string:Equals(text)
	return string.ToLower(self) == string.ToLower(text);
end

function string:Compare(text)
	local lowerSelf = string.ToLower(self);
	local lowerText = string.ToLower(text);
	if lowerSelf < lowerText then
		return -1;
	elseif lowerSelf == lowerText then
		return 0;
	else
		return 1;
	end
end

function string:StartsWith(text)
	if self == nil or text == nil or string.len(self) == 0 then
		return false;
	end
	
	local textLength = string.len(text);
	
	if textLength == 0 or textLength > string.len(self) then
		return false;
	end
	
	return string.ToLower(self:sub(1, textLength)) == string.ToLower(text);
end

function string:EndsWith(text)
	if self == nil or text == nil or string.len(self) == 0 then
		return false;
	end
	
	local textLength = string.len(text);
	local stringLength = string.len(self);
	
	if textLength == 0 or textLength > stringLength then
		return false;
	end
	
	return string.ToLower(self:sub(stringLength - textLength + 1)) == string.ToLower(text);
end

function string:Contains(text)
	if self == nil or text == nil or string.len(self) == 0 then
		return false;
	end
	
	local textLength = string.len(text);
	
	if textLength == 0 or textLength > string.len(self) then
		return false;
	end
	
	text = text:gsub("([%(%)%.%%%+%-%?%[%^%$%]])", "%%%1");
	
	return string.match(string.ToLower(self), string.ToLower(text));
end

function Round(amount, decimalPlaces)
  if decimalPlaces then
    return math.floor((amount * 10 ^ decimalPlaces) + 0.5) / (10 ^ decimalPlaces)
  else
    return math.floor(amount + 0.5)
  end
end

function FormatNumber(amount, decimalPlaces, negativePrefix)
  local str_amount,  formatted, famount, remain

  decimalPlaces = decimalPlaces or 2
  negativePrefix = negativePrefix or "-"

  famount = math.abs(Round(amount, decimalPlaces))
  famount = math.floor(famount)

  remain = Round(math.abs(amount) - famount, decimalPlaces)

  formatted = famount

  if decimalPlaces > 0 then
    remain = string.sub(tostring(remain),3)
    formatted = formatted .. Symbols["decimal_point"] .. remain .. string.rep("0", decimalPlaces - string.len(remain))
  end

  if amount < 0 then
    if negativePrefix == "()" then
      formatted = "(" .. formatted .. ")"
    else
      formatted = negativePrefix .. formatted 
    end
  end

  return formatted
end

function IsInRange(value, minimum, maximum)
	if value == nil then
		return false;
	end
	
	return value >= minimum and value <= maximum;
end

function IsInList(items, value)
	local found = false;
	for _, v in pairs(items) do
		if string.ToLower(v) == string.ToLower(value) then
			found = true;
			break
		end
	end
	
	return found;
end

function ToArray(items)
	local array = {};
	
	local index = 1;
	local done = false;
	while not done do
		local item = items[tostring(index)] or items[index];
		if item ~= nil then
			table.insert(array, item);
			index = index + 1;
		else
			done = true;
		end
	end
	
	return array;
end

function GetOpacityFromSettings(settings)
	local opacity = settings.Opacity;
	
	if type(opacity) == "number" and opacity <= 1 then
		return opacity;
	end
		
	return (tonumber(opacity) or 100) / 100;
end