import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Simbo.Poetical.Class"; -- Turbine library included so that there's no outside dependencies
import "Simbo.Poetical.Localisation";
import "Simbo.Poetical.Functions_Common";
import "Simbo.Poetical.Functions_Windows";
import "Simbo.Poetical.DropdownList";
import "Simbo.Poetical.CheckBoxList";
import "Simbo.Poetical.RadioButton";
import "Simbo.Poetical.ExpandCollapseButton";
import "Simbo.Poetical.ReadButton";
import "Simbo.Poetical.WordWrapper";
import "Simbo.Poetical";

Settings = 
{
	PoeticalWindow =
	{
		Position = {Left = "100", Top = "100"},
		Size = {Width = "400", Height = "400"},
		Visible = "yes",
		PoemsBoxHeight = "100",
		Opacity = "95"
	},
	EditorWindow =
	{
		Position = {Left = "150", Top = "150"},
		Size = {Width = "400", Height = "400"},
		Opacity = "95"
	},
	SettingsWindow =
	{
		Position = {Left = "200", Top = "200"},
		Size = {Width = "500", Height = "300"},
		Opacity = "95"
	},
	ToggleWindow =
	{
		Position = {Left = "100", Top = "100"},
		Opacity = "95"
	},
	SortOrder = "Asc",
	DefaultCommand = "",
	DefaultCommandForEachLine = true,
	BlankLineCommand = "",
	UseLocalBlankLineCommand = false,
	CommentPrefix = "%",
	Delay = "0",
	WordWrap = false,
	RememberPosition = true,
	MulitPrefix = ";;",
	ActionBarVisible = false
};

PoemDB = {
	["Poems"] = { }
};

Globals =
{
	Sizes =
	{
		Label = { Height = 24 },
		TextBox = { Width = 100, Height = 24 },
		WideTextBox = { Width = 150, Height = 24 },
		Button = { Width = 90, Height = 24 },
		WideButton = { Width = 125, Height = 24 },
		ActionBar = { Height = 24 }
	},
	Fonts = 
	{
		Label = Turbine.UI.Lotro.Font.Verdana14,
		SmallLabel = Turbine.UI.Lotro.Font.Verdana12, 
		TextBox = Turbine.UI.Lotro.Font.Verdana14,
		ListBox = Turbine.UI.Lotro.Font.Verdana14
	},
	Colours =
	{
		Label = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		SecondaryLabel = Turbine.UI.Color(1.0, 0.5, 0.5, 0.5),
		TextBox = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		ListBox = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		Confirmation = Turbine.UI.Color((210 / 255), 1, 0)
	},
	BackgroundColours =
	{
		TextBox = Turbine.UI.Color(1.0, 0, 0, 0),
		ListBox = Turbine.UI.Color(1.0, 0, 0, 0)
	},
	BorderColours =
	{
		ListBox = Turbine.UI.Color(1.0, 0.16, 0.19, 0.24)
	},
	Icons = 
	{
		BookmarkIconAdd = "Simbo/Poetical/Icons/Bookmark_Add.tga",
		BookmarkIconDelete = "Simbo/Poetical/Icons/Bookmark_Delete.tga",
		Divider = "Simbo/Poetical/Divider.tga",
		LineShortcut = "Simbo/Poetical/Icons/LineShortcut.tga",
		LineShortcut_Hover = "Simbo/Poetical/Icons/LineShortcut_Hover.tga",
		MainShortcut = "Simbo/Poetical/Icons/MainShortcut.tga",
		MainShortcut_Hover = "Simbo/Poetical/Icons/MainShortcut_Hover.tga",
		MainShortcut_Inactive = "Simbo/Poetical/Icons/MainShortcut_Inactive.tga",
		SortedAsc = "Simbo/Poetical/Icons/SortedAsc.tga",
		SortedAsc_Hover = "Simbo/Poetical/Icons/SortedAsc_Hover.tga",
		SortedDesc = "Simbo/Poetical/Icons/SortedDesc.tga",
		SortedDesc_Hover = "Simbo/Poetical/Icons/SortedDesc_Hover.tga",
		Toggle = "Simbo/Poetical/Icons/Toggle.tga",
		Toggle_Hover = "Simbo/Poetical/Icons/Toggle_Hover.tga",
		WindowResizer = "Simbo/Poetical/Icons/MouseDrag.tga"
	}
}

PoeticalLoad = Turbine.PluginData.Load;
PoeticalSave = Turbine.PluginData.Save;

Settings = PoeticalLoad(Turbine.DataScope.Account, "PoeticalSettings") or PoeticalLoad(Turbine.DataScope.Account, "Poetical") or Settings;
PoemDB = PoeticalLoad(Turbine.DataScope.Account, "PoeticalPoems") or PoeticalLoad(Turbine.DataScope.Character, "PoeticalData") or PoemDB;

SavePoems(true);

if Settings.WordWrap == nil then
	Settings.WordWrap = false;
end

if Settings.RememberPosition == nil then
	Settings.RememberPosition = true;
end

if Settings.MulitPrefix == nil then
	Settings.MulitPrefix = ";;";
end

if Settings.ActionBarVisible == nil then
	Settings.ActionBarVisible = false;
end

poeticalWindow = Simbo.Poetical.PoeticalWindow();
if Settings.PoeticalWindow.Visible == "yes" then
	poeticalWindow:SetVisible(true);
else
	poeticalWindow:SetVisible(false);
end

toggleWindow = Simbo.Poetical.ToggleWindow();
toggleWindow:SetVisible(true);
toggleWindow:SetZOrder(poeticalWindow:GetZOrder() + 3);
CheckWindowPosition(toggleWindow);

poeticalCommand = Turbine.ShellCommand();

function poeticalCommand:Execute(cmd, args)
	if args ~= nil then
		args = args:ToLower();
	end

	if args == Strings["cmd_show"] then
		poeticalWindow:Show();
	elseif args:sub(1, string.len(Strings["cmd_show"]) + 1) == (Strings["cmd_show"] .. " ") then
		poeticalWindow:OpenAtPoem(args:sub(string.len(Strings["cmd_show"]) + 2), false);
	elseif args:sub(1, 5) == "show " then
		poeticalWindow:OpenAtPoem(args:sub(6), false);
	elseif args == Strings["cmd_hide"] then
		poeticalWindow:Hide();
	elseif args == Strings["cmd_toggle"] then
		poeticalWindow:Toggle();
	else
		local popout = false;
		for i = 1, 4 do
			local popoutCommand = Strings["cmd_popout" .. tostring(i)];
			if popoutCommand ~= nil and args:sub(1, string.len(popoutCommand) + 1) == (popoutCommand .. " ") then
				poeticalWindow:OpenAtPoem(args:sub(string.len(popoutCommand) + 2), true);
				popout = true;
			end
		end
		if popout == false and args ~= nil then
			poeticalCommand:GetHelp();
		end
	end
end

function poeticalCommand:GetHelp()
	Turbine.Shell.WriteLine(Strings["usage"]);
	Turbine.Shell.WriteLine(" " .. Strings["cmd_help1"]);
	Turbine.Shell.WriteLine(" " .. Strings["cmd_help2"]);
	Turbine.Shell.WriteLine(" " .. Strings["cmd_help3"]);
end

Turbine.Shell.AddCommand("poetical", poeticalCommand );
Turbine.Shell.WriteLine("Poetical v"..Plugins["Poetical"]:GetVersion().." by Simbo");