import "Simbo.Poetical.ConfirmationWindow";

PoeticalListBox = class(Turbine.UI.Control);

local Constants =
{
	Border = 1,
	ScrollBarSize = 10,
	ActionBar = 
	{ 
		BookmarksWidth = 225,
		BookmarksDropdownHeight = 180,
		BookmarkIconSize = 18,
		Padding = 2;
	}
}

function PoeticalListBox:Constructor(parent, showButton, initialHeight, quickslot, doWordwrap, hasActionBar)
	Turbine.UI.Control.Constructor(self);

	self.Parent = parent;
	self.ShowButton = showButton;
	self.Quickslot = quickslot;
	self.LineCount = 0;
	self.SelectedIndex = -1;
	self.Lines = {};
	self.DoWordWrap = doWordwrap;
	self.Bookmarks = nil;
	self.HasActionBar = hasActionBar;
	self.MaxBookmarksHeightInRows = 10;
	
	self.SelectedIndexChanged = nil;
	
	self:SetParent(self.Parent);
	if initialHeight ~= nil then
		self:SetHeight(initialHeight);
	end
	
	self.AddBookmarkWindow = ConfirmationWindow("Small");
	self.AddBookmarkWindow:SetZOrder(self:GetZOrder() + 10);
	self.AddBookmarkWindow.ButtonClick = function (sender, args)
		if args.Text == "" then
			Turbine.Shell.WriteLine("<rgb=#FF0000>" .. Strings["invalid_bookmark"] .. "</rgb>");
		else
			AddBookmark(self.PoemId, self.Lines[self.SelectedIndex].PoemLineIndex, self.SelectedIndex, args.Text);
		end
	end
	
	self.ActionBar = Turbine.UI.Control();
	self.ActionBar:SetParent(self);
	self.ActionBar:SetHeight(Globals.Sizes.ActionBar.Height);
	self.ActionBar:SetPosition(0, 0);
	self.ActionBar:SetVisible(hasActionBar and Settings.ActionBarVisible);
	
	self.BookmarksDropdown = CreateDropdown(self.ActionBar, 0, 0, Constants.ActionBar.BookmarksWidth, Globals.Sizes.ActionBar.Height, Constants.ActionBar.BookmarksDropdownHeight);
	self.BookmarksDropdown:SetFont(Globals.Fonts.SmallListBox);
	self.BookmarksDropdown:SetHiddenFirstLine(true);
	self.BookmarksDropdown.SelectedValue = "";
	self.BookmarksDropdown.SelectedIndexChanged = function(sender, args)
		if self.BookmarksDropdown:GetSelectedIndex() > 1 and not self.BookmarksDropdown.DisableSelect then
			local selectedBookmark = args.SelectedValue;
			local selectIndex = tonumber(selectedBookmark.displayLine);
			if selectIndex ~= self.SelectedIndex then
				self:SelectLine(selectIndex, true);
			end
		end	
		
		self.BookmarksDropdown.DisableSelect = false;
	end
	
	self.BookmarkIconAdd = Turbine.UI.Control();
	self.BookmarkIconAdd:SetParent(self.ActionBar);
	self.BookmarkIconAdd:SetTop(2);
	self.BookmarkIconAdd:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self.BookmarkIconAdd:SetBackground(Globals.Icons.BookmarkIconAdd);
	self.BookmarkIconAdd:SetSize(Constants.ActionBar.BookmarkIconSize, Constants.ActionBar.BookmarkIconSize);
	self.BookmarkIconAdd:SetVisible(false);
	self.BookmarkIconAdd.MouseClick = function(sender, args)
		self.AddBookmarkWindow:ShowWindow(Strings["add_bookmark"], Strings["add_bookmark_message"], true, false);
	end
	
	self.BookmarkIconDelete = Turbine.UI.Control();
	self.BookmarkIconDelete:SetParent(self.ActionBar);
	self.BookmarkIconDelete:SetTop(2);
	self.BookmarkIconDelete:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self.BookmarkIconDelete:SetBackground(Globals.Icons.BookmarkIconDelete);
	self.BookmarkIconDelete:SetSize(Constants.ActionBar.BookmarkIconSize, Constants.ActionBar.BookmarkIconSize);
	self.BookmarkIconDelete:SetVisible(false);
	self.BookmarkIconDelete.MouseClick = function(sender, args) 
		DeleteBookmark(self.PoemId, self.Lines[self.SelectedIndex].PoemLineIndex, self.BookmarksDropdown:GetSelectedIndex() - 1);
	end
	
	self.ListBoxContainer = Turbine.UI.Control();
	self.ListBoxContainer:SetParent(self);
	self.ListBoxContainer:SetBackColor(Globals.BorderColours.ListBox);
	
	self.LinesBox = Turbine.UI.ListBox();
	self.LinesBox:SetParent(self.ListBoxContainer);
	self.LinesBox:SetPosition(Constants.Border, Constants.Border);
	self.LinesBox:SetBackColor(Globals.BackgroundColours.ListBox);
	self.LinesBox.ResetIndex = false;
	
	self.LinesScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.LinesScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.LinesScrollBar:SetParent(self.Parent);
	self.LinesBox:SetVerticalScrollBar(self.LinesScrollBar);

	self.LinesBox.SelectedIndexChanged = function(sender, args)
		local index = self.LinesBox:GetSelectedIndex();
		if index > 0 then
			if not self.LinesBox.ResetIndex then
				if self.LinesBox.QuickslotClicked == nil or not self.LinesBox.QuickslotClicked then
					self:SelectLine(index);
				end
			end
			self.LinesBox.ResetIndex = false;
			self.LinesBox:SetSelectedIndex(0);
		else
			self.LinesBox.ResetIndex = false;
		end
	end
	
	self.SizeChanged = function(sender, args)
		self.BookmarksDropdown:AdjustOpenPosition();
	end
	
	self.SelectedIndexChanged = function() end;
	
	self:ClearBookmarks();
end

function PoeticalListBox:Redraw()
	self:AdjustSizeAndPosition(self:GetLeft(), self:GetTop(), self:GetWidth() + Constants.ScrollBarSize, self:GetHeight());
end

function PoeticalListBox:AdjustHeight(height)
	self:AdjustSizeAndPosition(self:GetLeft(), self:GetTop(), self:GetWidth() + Constants.ScrollBarSize, height)
end

function PoeticalListBox:AdjustTopAndHeight(top, height)
	self:AdjustSizeAndPosition(self:GetLeft(), top, self:GetWidth() + Constants.ScrollBarSize, height)
end

function PoeticalListBox:AdjustSizeAndPosition(left, top, width, height)
	local bookmarkAdjust = 0;
	local showActionBar = self.HasActionBar and Settings.ActionBarVisible;
	
	self.ActionBar:SetVisible(showActionBar);
	
	if showActionBar then
		bookmarkAdjust = Globals.Sizes.ActionBar.Height - Constants.Border;
	end

	self:SetPosition(left, top);
	self:SetSize(width - Constants.ScrollBarSize, height);
	
	self.ActionBar:SetWidth(self:GetWidth());

	self.BookmarksDropdown:SetWidth(self:GetWidth() * 0.4);
	self.BookmarkIconAdd:SetLeft(self.BookmarksDropdown:GetWidth() + Constants.ActionBar.Padding);
	self.BookmarkIconDelete:SetLeft(self.BookmarksDropdown:GetWidth() + Constants.ActionBar.Padding);
	
	self.ListBoxContainer:SetTop(bookmarkAdjust);
	self.ListBoxContainer:SetSize(self:GetWidth(), self:GetHeight() - bookmarkAdjust);

	self.LinesBox:SetSize(self.ListBoxContainer:GetWidth() - Constants.Border * 2, self.ListBoxContainer:GetHeight() - Constants.Border * 2);
	
	self.LinesScrollBar:SetPosition(self:GetLeft() + self:GetWidth(), self:GetTop() + bookmarkAdjust);
	self.LinesScrollBar:SetSize(Constants.ScrollBarSize, self:GetHeight() - bookmarkAdjust);
	
	for i = 1, self.LinesBox:GetItemCount() do
		self.LinesBox:GetItem(i):LayoutControls();
	end
end

function PoeticalListBox:Clear()
	self.LinesBox:ClearItems();
	self.LineCount = 0;
	self.SelectedIndex = -1;
	self:EnableQuickSlot(false);
	
	self:ClearBookmarks();
	self.BookmarkIconAdd:SetVisible(false);
	self.BookmarkIconDelete:SetVisible(false);
	
	local index = table.getn(self.Lines);
	while index > 0 do
		self.Lines[index]:SetTextAndId("", "", nil, 0);
		self.Lines[index]:SetSelected(false);
		index = index - 1;
	end
end

function PoeticalListBox:AddLine(text, id, isSelectable, alias, colour, lineLength, noDelay, poemLineIndex, duplicateLine)
	local line = nil;

	if alias == nil then
		alias = text;
	end
	
	if isSelectable == nil then
		isSelectable = true;
	end
	
	if self.LineCount < table.getn(self.Lines) then
		line = self.Lines[self.LineCount + 1];
		line:SetSelectable(isSelectable);
		line:SetTextAndId(text, alias, id, lineLength);
		line:SetSelected(false);
	else
		line = PoeticalLine(self.LinesBox, lineLength, self.DoWordWrap, id, text, isSelectable, self.ShowButton, alias);
		table.insert(self.Lines, self.LineCount + 1, line);
	end
	
	line.NoDelay = noDelay;
	line.PoemLineIndex = poemLineIndex;
	line.DuplicateLine = duplicateLine;
	
	if colour ~= nil then
		line:SetColour(colour);
	else
		line:SetStandardColour(colour);
	end
	
	self.LinesBox:AddItem(line);
	line:LayoutControls();
	
	self.LineCount = self.LineCount + 1;
end

function PoeticalListBox:UpdateLine(index, text, alias, lineLength)
	if alias == nil then
		alias = text;
	end
	
	self.LinesBox:GetItem(index):SetTextOnly(text, alias, lineLength);
end

function PoeticalListBox:SelectLineById(id, scrollToTop)
	local index = 1;
	local checkLine;
	local found = false;
	
	while index <= self.LineCount and not found do
		checkLine = self.LinesBox:GetItem(index);
		if checkLine.Id == id then
			found = true;
		else
			index = index + 1;
		end
	end
	
	if found then
		self:SelectLine(index, scrollToTop);
	end
end

function PoeticalListBox:SelectLine(index, scrollToTop)
	local selectedLine = nil;
	local foundLine = false;

	if index > self.LineCount then
		index = 0;
	end

	selectedLine = self:HighlightLine(index);
	
	local selectedId;
	if selectedLine ~= nil then
		selectedId = selectedLine.Id;
		
		if scrollToTop ~= nil and scrollToTop then
			if self.SelectedIndex > 0 then
				self.ResetIndex = true;
				self.LinesBox:SetSelectedIndex(self.SelectedIndex);
			end
			--self.LinesBox:SetSelectedIndex(0);
		end
	else
		self:EnableQuickSlot(false);
	end
	
	local args = 
	{
		SelectedIndex = self.SelectedIndex,
		SelectedId = selectedId  
	}; 
	
	self.SelectedIndexChanged(self, args);
end

function PoeticalListBox:SelectNextLine(fromIndex)
	local index = fromIndex + 1;
	local checkLine;
	local found = false;
	
	while index <= self.LineCount and not found do
		checkLine = self.LinesBox:GetItem(index);
		if checkLine.IsSelectable then
			found = true;
		else
			index = index + 1;
		end
	end
	
	self:SelectLine(index);
	self:ScrollIntoView();
end

function PoeticalListBox:HighlightLine(index)
	local selectedLine = nil;
	
	if index >= 1 then
		selectedLine = self.LinesBox:GetItem(index);
	end
	
	if selectedLine == nil or selectedLine.IsSelectable then
		self.BookmarkIconDelete:SetVisible(false);
		self.BookmarkIconAdd:SetVisible(false);
	
		if self.SelectedIndex >= 1 then
			self.LinesBox:GetItem(self.SelectedIndex):SetSelected(false);
		end

		local isNewLine = (self.SelectedIndex ~= index);
		self.SelectedIndex = index;
		
		if selectedLine ~= nil then
			if self.Bookmarks ~= nil then
				if isNewLine then
					self.AddBookmarkWindow:Hide();
				end
				
				local bookmarkCount = #self.Bookmarks;
				local bookmarkIndex = 1;
				local bookmark = nil;
				local bookmarkFound = false;
				
				while bookmarkIndex <= bookmarkCount and not bookmarkFound do
					bookmark = self.Bookmarks[bookmarkIndex];
					if bookmark.displayLine ~= self.SelectedIndex then
						bookmarkIndex = bookmarkIndex + 1;
					else
						bookmarkFound = true;
					end
				end

				if bookmarkFound then
					bookmarkIndex = bookmarkIndex + 1;
					self.BookmarkIconDelete:SetVisible(true);
				else 
					bookmarkIndex = 1;
					self.BookmarkIconAdd:SetVisible(not selectedLine.DuplicateLine);
				end
				
				if self.BookmarksDropdown:GetSelectedIndex() ~= bookmarkIndex then
					self.BookmarksDropdown.DisableSelect = true;
					self.BookmarksDropdown:SetSelectedIndex(bookmarkIndex);
				end
			end
			
			selectedLine:SetSelected(true);
			if self.Quickslot ~= nil then
				--local shortcut = self.Quickslot:GetShortcut();
				--shortcut:SetData(selectedLine:GetAlias());
				self.Quickslot:SetShortcut(selectedLine:GetAlias());
				self:EnableQuickSlot(true, selectedLine.NoDelay);
			end
		end
	end
	
	return selectedLine;
end

function PoeticalListBox:GetSelectedLineIndex()
	return self.SelectedIndex;
end

function PoeticalListBox:GetSelectedLineId()
	if self.SelectedIndex >= 1 then
		return self.LinesBox:GetItem(self.SelectedIndex).Id;
	else
		return nil;
	end
end

function PoeticalListBox:GetIndexForId(id)
	local selectedIndex = nil;
	local index = 1;
	
	while selectedIndex == nil and index <= self.LineCount do
		if self.LinesBox:GetItem(index).Id == id then
			selectedIndex = index;
		end
		index = index + 1;
	end
	
	return selectedIndex;
end

function PoeticalListBox:EnableQuickSlot(enabled, noDelay)
	if self.Quickslot ~= nil then
		self.Quickslot:SetEnabled(enabled, nil, noDelay);
	end
end

function PoeticalListBox:ScrollIntoView(lineIndex)
	if lineIndex == nil then
		lineIndex = self.SelectedIndex;
	end
	
	if lineIndex > 0 then
		local item = self.LinesBox:GetItemAt(0, 16);
		if item ~= nil then
			local topIndex = self.LinesBox:IndexOfItem(item);
			local newIndex = 0;
			if topIndex > lineIndex then
				newIndex = lineIndex;
			else
				item = self.LinesBox:GetItemAt(0, self.LinesBox:GetHeight() - 16);
				local bottomIndex = self.LinesBox:IndexOfItem(item);
				if bottomIndex < lineIndex then
					newIndex = lineIndex - (bottomIndex - topIndex);
				end
			end
			
			if newIndex > 0 then
				if newIndex > self.LinesBox:GetItemCount() then
					newIndex = self.LinesBox:GetItemCount();
				end
				self.LinesBox.ResetIndex = true;
				self.LinesBox:SetSelectedIndex(newIndex);
			end
		end
	end
end

function PoeticalListBox:ScrollToTop()
	self.LinesBox.ResetIndex = true;
	self.LinesBox:SetSelectedIndex(1);
end

function PoeticalListBox:SetWordWrap(doWordWrap)
	if doWordWrap ~= self.DoWordWrap then
		self.DoWordWrap = doWordWrap;
		local totalLines = table.getn(self.Lines);
		local index = 1;
		while index <= totalLines do
			self.Lines[index]:SetWordWrap(doWordWrap);
			index = index + 1;
		end
	end
end

function PoeticalListBox:GetLineCount()
	return self.LineCount;
end

function PoeticalListBox:SetBookmarkInfo(bookmarks, poemId)
	self.Bookmarks = bookmarks;
	self.PoemId = poemId;
	
	self.BookmarksDropdown:ClearItems();
	self.BookmarksDropdown:AddItem(Strings["select_bookmark"], "");
	
	for i, v in ipairs(self.Bookmarks) do
		self.BookmarksDropdown:AddItem(v.name, v);
	end
	
	self:SetBookmarksHeightInRows();
	self.BookmarksDropdown:SetSelectedIndex(1);
	
	self:Redraw();
end

function PoeticalListBox:ClearBookmarks()
	self:SetBookmarkInfo({}, nil);

	self.AddBookmarkWindow:Hide();
end

function PoeticalListBox:RemoveBookmarkIndex(bookmarkIndex)
	local currentBookmark = self.Bookmarks[bookmarkIndex];
	
	table.remove(self.Bookmarks, bookmarkIndex);
	self.BookmarksDropdown:RemoveItemAt(bookmarkIndex + 1, 1);
	
	if self.SelectedIndex == currentBookmark.displayLine then
		self.BookmarkIconDelete:SetVisible(false);
		self.BookmarkIconAdd:SetVisible(true);
	end
end

function PoeticalListBox:AddBookmarkIndex(bookmark)
	local index = 1;

	if #self.Bookmarks > 0 then
		while index <= #self.Bookmarks and bookmark.actualLine > self.Bookmarks[index].actualLine do
			index = index + 1;
		end
	end
	
	table.insert(self.Bookmarks, index, bookmark);
	self.BookmarksDropdown:InsertItemAt(index + 1, bookmark.name, bookmark, true);
	self:SetBookmarksHeightInRows();
	
	if self.SelectedIndex == bookmark.displayLine then
		self.BookmarkIconDelete:SetVisible(true);
		self.BookmarkIconAdd:SetVisible(false);
	end
end

function PoeticalListBox:SetBookmarksHeightInRows()
	local bookmarkHeightInRows = #self.Bookmarks;
	
	if #self.Bookmarks > self.MaxBookmarksHeightInRows then
		bookmarkHeightInRows = self.MaxBookmarksHeightInRows;
	end
	
	if bookmarkHeightInRows == 0 then
		bookmarkHeightInRows = 10
	end
	
	self.BookmarksDropdown:SetHeightInRows(bookmarkHeightInRows);
end