import "Simbo.Poetical.PoeticalListBox";
import "Simbo.Poetical.PoeticalLine";

PoeticalWindow = class(WindowLotro);

local Constants =
{
	MinimumSize =
	{
		Width = 400,
		Height = 400
	},
	Buttons =
	{
		Padding = { Left = 15, Top = 45, Bottom = 20, Right = 25 },
		Spacing = 10
	},
	Filter =
	{
		Padding = { Left = 15, Top = 95, Bottom = 20 },
		Size = { Width = 180, Height = 24, DropdownHeight = 180 },
		Spacing = 10
	},
	Divider =
	{
		Width = 99,
		Height = 9
	},
	SortIcon =
	{
		Width = 15,
		Height = 15
	},
	ListBoxes = 
	{	
		Padding = { Left = 15, Top = 125, Right = 15, Bottom = 80},
		Spacing = 10,
		MinimumHeight = 42
	}
}

local SaveLines = {};
local WordWrapInfo = {};
local UnfilteredPoems = {};
local ShortcutImages = {};

local BookmarkPrefix = "{{BOOKMARK:"
local BookmarkSuffix = "}}"
local BookmarkPatten = "^" .. BookmarkPrefix .. "(.*)" .. BookmarkSuffix;

function PoeticalWindow:Constructor()
	WindowLotro.Constructor(self);
	
	self.EditPoemId = nil;
	self.DeletePoemId = nil;
	self.SelectedPoemId = nil;
	self.Popups = {};
	
	self.WordWrapper = WordWrapper(self);
	
	Settings.SortOrder = Settings.SortOrder or "Asc";

	self:SetPosition(tonumber(Settings.PoeticalWindow.Position.Left) or 100, tonumber(Settings.PoeticalWindow.Position.Top) or 100);
	self:SetSize(tonumber(Settings.PoeticalWindow.Size.Width) or Constants.MinimumSize.Width, tonumber(Settings.PoeticalWindow.Size.Height) or Constants.MinimumSize.Height);
	if self:GetWidth() < Constants.MinimumSize.Width then
		self:SetWidth(Constants.MinimumSize.Width);
	end
	if self:GetHeight() < Constants.MinimumSize.Height then
		self:SetHeight(Constants.MinimumSize.Height);
	end
	self:SetText(Strings["poetical"]);
	self:SetOpacity(GetOpacityFromSettings(Settings.PoeticalWindow));
	self:SetResizer(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
	
	self.EditorWindow = Simbo.Poetical.EditorWindow();
	self.EditorWindow:SetVisible(false);

	self.EditorWindow.Saved = function(sender, args)
		self:SavePoem(args.Poem, self.EditPoemId);
	end

	self.SettingsWindow = Simbo.Poetical.SettingsWindow();
	self.SettingsWindow:SetVisible(false);
	
	local previousWordWrap = nil;
	local previousHasActionBar = nil;
	local previousRememberPosition = nil;
	
	self.SettingsWindow.Saved = function(sender, args)
		if args.Changed then
			ClearWordWrapInfo();
			
			if Settings.WordWrap ~= previousWordWrap then
				self.LinesControl:SetWordWrap(Settings.WordWrap);
			end
			
			if Settings.ActionBarVisible ~= previousHasActionBar then
				self.LinesControl:Redraw();
			end
			
			if Settings.RememberPosition ~= previousRememberPosition then
				if self.SelectedPoemId ~= nil then
					local current = SaveLines[self.SelectedPoemId];
					for index, value in pairs(SaveLines) do
						SaveLines[index] = nil;
					end
					SaveLines[self.SelectedPoemId] = current;
				end
			end
			
			if self.SelectedPoemId ~= nil then
				self:LoadPoem(self.SelectedPoemId, false);
			end
			
			for index, popup in pairs(self.Popups) do
				if popup:IsVisible() then
					popup.LinesControl:SetWordWrap(Settings.WordWrap);
					popup:RefreshPoem(true);
				end
			end
			
			if self.EditorWindow:IsVisible() then
				self.EditorWindow:ShowDefaultCommands();
			end
		end
	end
	
	local confirmationWindow = Simbo.Poetical.ConfirmationWindow();
	confirmationWindow:SetZOrder(self:GetZOrder() + 10);
	confirmationWindow.ButtonClick = function (sender,args)
		if confirmationWindow.Tag == "DELETE" then
			self:DeleteSelectedPoem();
		elseif confirmationWindow.Tag == "NEW" then
			self:ShowNewPoemWindow();
		elseif confirmationWindow.Tag == "EDIT" then
			self:ShowEditPoemWindow();
		end
	end
			
	local buttonPosition = Constants.Buttons.Padding.Left;
	local nextButton = Globals.Sizes.Button.Width + Constants.Buttons.Spacing;
	
	local newButton = CreateButton(self);
	newButton:SetPosition(Constants.Buttons.Padding.Left, Constants.Buttons.Padding.Top);
	newButton:SetText(Strings["new"]);
	newButton.MouseClick = function(sender, args)
		if(args.Button == Turbine.UI.MouseButton.Left) then	
			if self.EditorWindow:IsVisible() then
				confirmationWindow.Tag = "NEW";
				confirmationWindow:ShowWindow(Strings["new_poem"], Strings["poem_window_open"]);
			else
				self:ShowNewPoemWindow();
			end
		end
	end	
	
	buttonPosition = buttonPosition + nextButton;
	
	self.EditButton = CreateButton(self);
	self.EditButton:SetPosition(buttonPosition, Constants.Buttons.Padding.Top);
	self.EditButton:SetText(Strings["edit"]);
	self.EditButton:SetEnabled(false);
	self.EditButton.MouseClick = function(sender, args)
		if(args.Button == Turbine.UI.MouseButton.Left) then
			if self.EditorWindow:IsVisible() then
				confirmationWindow.Tag = "EDIT";
				confirmationWindow:ShowWindow(Strings["new_poem"], Strings["poem_window_open"]);
			else
				self:ShowEditPoemWindow();
			end
		end
	end	
	
	buttonPosition = buttonPosition + nextButton;
	
	self.DeleteButton = CreateButton(self);
	self.DeleteButton:SetPosition(buttonPosition, Constants.Buttons.Padding.Top);
	self.DeleteButton:SetText(Strings["delete"]);
	self.DeleteButton:SetEnabled(false);
	self.DeleteButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			if self.SelectedPoemId ~= nil then
				self.DeletePoemId = self.SelectedPoemId;
				confirmationWindow.Tag = "DELETE";
				confirmationWindow:ShowWindow(Strings["delete_poem_with_separator"] .. PoemDB.Poems[self.SelectedPoemId].Title, Strings["delete_confirmation"]);
			end
		end
	end	
	
	local left = Constants.Filter.Padding.Left;
	local tagsLabel = CreateLabel(self);
	tagsLabel:SetPosition(left, Constants.Filter.Padding.Top - tagsLabel:GetHeight());
	tagsLabel:SetWidth(Globals.Sizes.WideTextBox.Width);
	tagsLabel:SetText(Strings["tags"])
	
	self.TagsDropdown = CreateDropdown(self, left, Constants.Filter.Padding.Top, Constants.Filter.Size.Width, Constants.Filter.Size.Height, Constants.Filter.Size.DropdownHeight);
	self.TagsDropdown:SetFont(Globals.Fonts.SmallListBox);
	self.TagsDropdown.SelectedValue = "";
	self:PopulateTagsDropdown(false);
	
	self.TagsDropdown.SelectedIndexChanged = function(sender, args)
		local value = args.SelectedValue;

		if value ~= self.TagsDropdown.SelectedValue then
			self.TagsDropdown.SelectedValue = value;
			self:ClearLines();
			self:LoadAllPoems(true);
		end
	end
	
	left = Constants.Filter.Padding.Left + Constants.Filter.Size.Width + Constants.Filter.Spacing;
	local keywordLabel = CreateLabel(self);
	keywordLabel:SetPosition(left, Constants.Filter.Padding.Top - keywordLabel:GetHeight());
	keywordLabel:SetText(Strings["filter"]);
	
	self.KeywordFilter = CreateWideTextBox(self, false);
	self.KeywordFilter:SetPosition(left, Constants.Filter.Padding.Top);
	self.KeywordFilter.FocusGained = function (sender, args)
		self.KeywordFilter.SaveText = self.KeywordFilter:GetText();
		self.KeywordFilter:SetWantsUpdates(true);
	end
	self.KeywordFilter.FocusLost = function (sender, args)
		self.KeywordFilter:SetWantsUpdates(false);
	end
	self.KeywordFilter.Update = function (sender, args)
		local text = sender:GetText();
		if text ~= sender.SaveText then
			sender.SaveText = text;
			self:FilterPoemsByKeyword(text);
		end
	end
	
	self.SortIcon = Turbine.UI.Control();
	self.SortIcon:SetParent(self);
	self.SortIcon:SetSize(Constants.SortIcon.Width, Constants.SortIcon.Height);
	self.SortIcon.SetIcon = function(hover)
		local icon;
		
		if Settings.SortOrder == "Asc" then
			if hover then
				icon = "SortedAsc_Hover";
			else
				icon = "SortedAsc";
			end
		else
			if hover then
				icon = "SortedDesc_Hover";
			else
				icon = "SortedDesc";
			end
		end
		SetIconImage(ShortcutImages, self.SortIcon, icon);
	end
	self.SortIcon.SetIcon(false);
	self.SortIcon.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			Settings.SortOrder = (Settings.SortOrder == "Asc" and "Desc" or "Asc");
			self.SortIcon.SetIcon(true);
			self:LoadAllPoems(true);
		end
	end	
	self.SortIcon.MouseEnter = function(sender,args)
		self.SortIcon.SetIcon(true);
	end
	self.SortIcon.MouseLeave = function(sender,args)
		self.SortIcon.SetIcon(false);
	end
	
	self.Quickslot = ReadButton(self);
	self.Quickslot.ButtonPressed = function(sender, args)
		self.LinesControl:SelectNextLine(SaveLines[self.SelectedPoemId]);
	end
	
	self.PoemsControl = PoeticalListBox(self, false, tonumber(Settings.PoeticalWindow.PoemsBoxHeight) or Constants.ListBoxes.MinimumHeight, nil, false, false);
	self.Divider = CreateDivider(self);
	self.LinesControl = PoeticalListBox(self, true, nil, self.Quickslot, Settings.WordWrap, true);
	
	self.DefaultPrefixLabel = CreateLabel(self);
	
	self.SettingsButton = CreateButton(self, UIOptions["wide_settings_button"]);
	self.SettingsButton:SetText(Strings["settings"]);
	self.SettingsButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			if not self.SettingsWindow:IsVisible() then
				previousWordWrap = Settings.WordWrap;
				previousHasActionBar = Settings.HasActionBar;
				previousRememberPosition = Settings.RememberPosition;
				self.SettingsWindow:ShowWindow();
			end
			self.SettingsWindow.Activate();
		end
	end	
	
	self.PopoutButton = CreateButton(self, UIOptions["wide_popout_button"]);
	self.PopoutButton:SetText(Strings["popout"]);
	self.PopoutButton:SetEnabled(false);
	self.PopoutButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			local poemId = self.SelectedPoemId;
			self:PopoutPoem(poemId);
		end
	end	
	
	self:LayoutControls();
		
	self.PoemsControl.SelectedIndexChanged = function(sender, args)
		local enabled = args.SelectedIndex >= 1;
		self.EditButton:SetEnabled(enabled);
		self.DeleteButton:SetEnabled(enabled);
		self.PopoutButton:SetEnabled(enabled);
		
		local newPoemSelected = (self.SelectedPoemId ~= args.SelectedId);
		self.SelectedPoemId = args.SelectedId;
		if not Settings.RememberPosition then
			SaveLines[self.SelectedPoemId] = nil;
		end
		self:LoadPoem(args.SelectedId, newPoemSelected);
	end
	
	self.LinesControl.SelectedIndexChanged = function(sender, args)
		SaveLines[self.SelectedPoemId] = args.SelectedIndex;
	end

	self.VisibleChanged = function(sender, args)
		if not self.UIHidden then
			if self:IsVisible() then
				Settings.PoeticalWindow.Visible = "yes";
			else
				Settings.PoeticalWindow.Visible = "no";
			end
		
			if self:IsVisible() == false then
				HideChildWindow(self.EditorWindow);
				HideChildWindow(self.SettingsWindow);
				self:SaveSettings(false);
				
				if self.Popups ~= nil then
					for index, popup in pairs(self.Popups) do
						popup:Close();
					end
				end
			end
		end
		
		if self:IsVisible() then
			CheckWindowPosition(self);
		end
	end
	
	if (Plugins ["Poetical"] ~= nil ) then
		Plugins["Poetical"].Unload = function( sender, args )
			self:SaveSettings(true);
		end
	end
	
	self:LoadAllPoems(false);
end

function CreateDivider(parent)
	local divider = Turbine.UI.Control();
	divider.Parent = parent;
	divider:SetParent(parent);
	divider:SetBackground(Globals.Icons.Divider);
	divider:SetSize(Constants.Divider.Width, Constants.Divider.Height);
	
	divider.MouseDown = function(sender,args)
		sender.DragStartY = args.Y;
		sender.Dragging = true;
	end
		
	divider.MouseUp = function(sender,args)
	  sender.Dragging = false;
	end
	
	divider.MouseMove = function(sender, args)
		if (sender.Dragging) then
			local adjust = args.Y - sender.DragStartY;
			local parent = sender.Parent;
			local adjustForActionBar = 0;
			
			if Settings.ActionBarVisible then
				adjustForActionBar = Globals.Sizes.ActionBar.Height;
			end
			
			if parent.PoemsControl:GetHeight() + adjust < Constants.ListBoxes.MinimumHeight then
				adjust = Constants.ListBoxes.MinimumHeight - parent.PoemsControl:GetHeight();
			elseif parent.LinesControl:GetHeight() - adjust < Constants.ListBoxes.MinimumHeight + adjustForActionBar then
				adjust = parent.LinesControl:GetHeight() - Constants.ListBoxes.MinimumHeight - adjustForActionBar;
			end
			
			parent.PoemsControl:AdjustHeight(parent.PoemsControl:GetHeight() + adjust);
			sender:SetTop(sender:GetTop() + adjust);
			parent.LinesControl:AdjustTopAndHeight(parent.LinesControl:GetTop() + adjust, parent.LinesControl:GetHeight() - adjust);
		end
	end
	
	return divider;
end

function PoeticalWindow:LayoutControls()
	self.SortIcon:SetPosition(self:GetWidth() - Constants.ListBoxes.Padding.Left - Constants.SortIcon.Width - 10, Constants.Filter.Padding.Top + Constants.Filter.Size.Height - Constants.SortIcon.Height);
	
	local adjustForActionBar = 0;
	if Settings.ActionBarVisible then
		adjustForActionBar = Globals.Sizes.ActionBar.Height;
	end
	
	local poemsBoxHeight = self.PoemsControl:GetHeight();
	local checkHeight = self:GetHeight() - Constants.ListBoxes.Padding.Top - Constants.ListBoxes.Padding.Bottom - self.PoemsControl:GetHeight() - Constants.ListBoxes.Spacing;
	if checkHeight < Constants.ListBoxes.MinimumHeight + adjustForActionBar then
		poemsBoxHeight = self:GetHeight() - Constants.ListBoxes.Padding.Top - Constants.ListBoxes.Padding.Bottom - Constants.ListBoxes.Spacing - Constants.ListBoxes.MinimumHeight - adjustForActionBar;
	end

	self.PoemsControl:AdjustSizeAndPosition(Constants.ListBoxes.Padding.Left, Constants.ListBoxes.Padding.Top, self:GetWidth() - Constants.ListBoxes.Padding.Left - Constants.ListBoxes.Padding.Right, poemsBoxHeight);
	self.Divider:SetPosition((self:GetWidth() - Constants.Divider.Width) / 2, Constants.ListBoxes.Padding.Top + poemsBoxHeight + (Constants.ListBoxes.Spacing - Constants.Divider.Height) / 2);
	self.LinesControl:AdjustSizeAndPosition(Constants.ListBoxes.Padding.Left, Constants.ListBoxes.Padding.Top + poemsBoxHeight + Constants.ListBoxes.Spacing, self:GetWidth() - Constants.ListBoxes.Padding.Left - Constants.ListBoxes.Padding.Right, self:GetHeight() - Constants.ListBoxes.Padding.Top - Constants.ListBoxes.Padding.Bottom - self.PoemsControl:GetHeight() - Constants.ListBoxes.Spacing);
	
	self.DefaultPrefixLabel:SetPosition(Constants.ListBoxes.Padding.Left, self.LinesControl:GetTop() + self.LinesControl:GetHeight());
	self.DefaultPrefixLabel:SetWidth(self:GetWidth() - Constants.ListBoxes.Padding.Left - Constants.ListBoxes.Padding.Right);
	
	self.SettingsButton:SetPosition(Constants.Buttons.Padding.Left, self:GetHeight() - Constants.Buttons.Padding.Bottom - self.SettingsButton:GetHeight());
	self.PopoutButton:SetPosition(self:GetWidth() - self.PopoutButton:GetWidth() - Constants.Buttons.Padding.Right, self:GetHeight() - Constants.Buttons.Padding.Bottom - self.PopoutButton:GetHeight());
	
	self.Quickslot:SetPosition((self:GetWidth() - self.Quickslot:GetWidth()) / 2, self:GetHeight() - Constants.Buttons.Padding.Bottom - self.Quickslot:GetHeight());
end

function PoeticalWindow:Destructor()
	self.LinesControl:Clear();
	self.PoemsControl:Clear();

	self.Popups:Clear();
end

function PoeticalWindow:DeleteSelectedPoem()
	self:DeletePoem(self.DeletePoemId);
	if self.DeletePoemId == self.EditPoemId then
		self.EditorWindow.CancelPressed = true;
		self.EditorWindow:Close();
	end
	if self.Popups[self.DeletePoemId] ~= nil then
		self.Popups[self.DeletePoemId]:Close();
		self.Popups[self.DeletePoemId] = nil;
	end
end

function PoeticalWindow:ShowNewPoemWindow()
	self.EditPoemId = nil;
	self.EditorWindow:ShowWindow(nil);
end

function PoeticalWindow:ShowEditPoemWindow()
	if self.SelectedPoemId ~= nil then
		if not self.EditorWindow:IsVisible() or self.EditPoemId ~= self.SelectedPoemId then
			self.EditPoemId = self.SelectedPoemId;
			self.EditorWindow:ShowWindow(PoemDB.Poems[self.SelectedPoemId]);
		else
			self.EditorWindow:ActivateWindow();
		end
	end
end

function PoeticalWindow:Show()
	if not self:IsVisible() then
		ShowChildWindow(self.EditorWindow);
		ShowChildWindow(self.SettingsWindow);
		self:SetVisible(true);
	end
	self:Activate();
end

function ShowChildWindow(childWindow)
	if childWindow.ReopenOnToggle then
		childWindow.ReopenOnToggle = false;
		childWindow:SetVisible(true);
	end
end

function PoeticalWindow:Hide()
	if self:IsVisible() then
		HideChildWindow(self.EditorWindow);
		HideChildWindow(self.SettingsWindow);
		self:SetVisible(false);
	end
end

function HideChildWindow(childWindow)
	if childWindow:IsVisible() then
		childWindow.ReopenOnToggle = true;
		childWindow:SetVisible(false);
	end
end

function PoeticalWindow:Toggle()
	if self:IsVisible() then
		self:Hide();
	else
		self:Show();
	end
end

function PoeticalWindow:SetPopoutOpacity(opacity)
	if self.Popups ~= nil then
		for index, popup in pairs(self.Popups) do
			popup:SetOpacity(opacity);
		end
	end
end

function PoeticalWindow:PopoutPoem(poemId)
	if poemId ~= nil then
		if self.Popups[poemId] == nil then
			self.Popups[poemId] = Simbo.Poetical.PopoutWindow(self);
		end
		self.Popups[poemId]:ShowWindow(poemId);
	end
end

function PoeticalWindow:OpenAtPoem(poemName, asPopout)
	if not self:IsVisible() then
		self:SetVisible(true);
		self:Activate();
	end
	
	poemName = poemName:Trim();
	if poemName:StartsWith("\"") and poemName:EndsWith("\"") then
		poemName = poemName:sub(2, string.len(poemName) - 1);
	end
	
	local foundPoemId = nil;
	local foundPoem = nil;
	
	for poemId, poem in pairs(PoemDB.Poems) do
		if poem.Title:Equals(poemName) then
			foundPoemId = poemId;
			foundPoem = poem;
			break;
		end
	end
	
	if foundPoemId == nil then
		return;
	end
	
	if asPopout then
		self:PopoutPoem(foundPoemId);
		return;
	end
	
	local filter = self.TagsDropdown.SelectedValue;
	if filter ~= nil and filter ~= "" then
		if foundPoem.Tags ~= nil and not IsInList(foundPoem.Tags, filter) then
			self.TagsDropdown:SetSelectedIndex(1, false);
			self.TagsDropdown.SelectedValue = "";
			self:LoadAllPoems(false);
		end
	end
	
	self.PoemsControl:SelectLineById(foundPoemId, true);
end

function PoeticalWindow:SaveSettings(showMessage)
	Settings.PoeticalWindow.Position.Top = tostring(self:GetTop());
	Settings.PoeticalWindow.Position.Left = tostring(self:GetLeft());
	Settings.PoeticalWindow.Size.Width = tostring(self:GetWidth());
	Settings.PoeticalWindow.Size.Height = tostring(self:GetHeight());
	Settings.PoeticalWindow.PoemsBoxHeight = tostring(self.PoemsControl:GetHeight());
	
	Settings.ToggleWindow.Position.Left = tostring(toggleWindow:GetLeft());
	Settings.ToggleWindow.Position.Top = tostring(toggleWindow:GetTop());		
			
	Settings.EditorWindow.Position.Top = tostring(self.EditorWindow:GetTop());
	Settings.EditorWindow.Position.Left = tostring(self.EditorWindow:GetLeft());
	Settings.EditorWindow.Size.Width = tostring(self.EditorWindow:GetWidth());
	Settings.EditorWindow.Size.Height = tostring(self.EditorWindow:GetHeight());
	
	Settings.SettingsWindow.Position.Top = tostring(self.SettingsWindow:GetTop());
	Settings.SettingsWindow.Position.Left = tostring(self.SettingsWindow:GetLeft());
	Settings.SettingsWindow.Size.Width = tostring(self.SettingsWindow:GetWidth());
	Settings.SettingsWindow.Size.Height = tostring(self.SettingsWindow:GetHeight());

	if showMessage then
		PoeticalSave(Turbine.DataScope.Account, "PoeticalSettings", Settings,
			function(result, message)
				if result then
					Turbine.Shell.WriteLine("<rgb=#00FF00>" .. Strings["settings_saved"] .. "</rgb>");
				else
					Turbine.Shell.WriteLine("<rgb=#FF0000>" .. Strings["settings_notsaved"] .. " " .. message .. "</rgb>");
				end
			end);
	else
		PoeticalSave(Turbine.DataScope.Account, "PoeticalSettings", Settings);
	end
end

function PoeticalWindow:LoadAllPoems(reselectPoem)
	local selectedPoemId =  self.SelectedPoemId;
	local filter = self.TagsDropdown.SelectedValue;
	local filterText = self.KeywordFilter:GetText();
	
	self.EditButton:SetEnabled(false);
	self.DeleteButton:SetEnabled(false);
	self.PopoutButton:SetEnabled(false);
	
	self.PoemsControl:Clear();
	for key in ipairs(UnfilteredPoems) do
		UnfilteredPoems[key] = nil
	end
	
	local sortedPoems = {}
    for poemId, poem in pairs(PoemDB.Poems) 
	do 
		local addPoem = true;
		if filter ~= nil and filter ~= "" then
			if poem.Tags == nil or not IsInList(poem.Tags, filter) then
				addPoem = false;
			end
		end
		
		if addPoem then
			table.insert(sortedPoems, poem);
		end
	end
	table.sort(sortedPoems, function(poemA, poemB)
		if Settings.SortOrder == "Asc" then
			return poemA.Title < poemB.Title
		else
			return poemA.Title > poemB.Title
		end
	end)
	
	local index = 1;
	for poemId, poem in pairs(sortedPoems) 
	do
		if string.len(filterText) == 0 or poem.Title:Contains(filterText) then
			self.PoemsControl:AddLine(poem.Title, poem.Id);
		end
		
		UnfilteredPoems[index] = poem;
		index = index + 1;
	end
	
	if reselectPoem then
		local reselectPoemIndex = self.PoemsControl:GetIndexForId(selectedPoemId);
		if reselectPoemIndex ~= nil then
			self.PoemsControl:SelectLine(reselectPoemIndex);
		else
			self.SelectedPoemId = nil;
		end
	else
		self.SelectedPoemId = nil;
	end
end

function PoeticalWindow:FilterPoemsByKeyword(keyword)
	self.PoemsControl:Clear();
	
	local newIndex = 1;
	local reselectPoemIndex = 0;
	for index, poem in ipairs(UnfilteredPoems) 
	do 
		if string.len(keyword) == 0 or poem.Title:Contains(keyword) then
			self.PoemsControl:AddLine(poem.Title, poem.Id);
			if self.SelectedPoemId ~= nil and poem.Id == self.SelectedPoemId then
				reselectPoemIndex = newIndex;
			end
			newIndex = newIndex + 1;
		end
	end
	
	if reselectPoemIndex > 0 then
		self.PoemsControl:SelectLine(reselectPoemIndex);
	else
		self:ClearLines();
		self.SelectedPoemId = nil;
	end
end

function PoeticalWindow:LoadPoem(poemId, scrollLineToTop)
	PopulatePoeticalListBox(poemId, self.LinesControl, self.DefaultPrefixLabel);
	
	if poemId ~= nil then
		if SaveLines[poemId] ~= nil then
			if scrollLineToTop == nil then
				scrollLineToTop = true;
			end
			self.LinesControl:SelectLine(SaveLines[poemId], scrollLineToTop);
		else
			if scrollLineToTop then
				self.LinesControl:ScrollToTop();
			end
		end
	end
end

function PoeticalWindow:SavePoem(poemToSave, poemId)
	local isNewPoem = false;
	
	if poemId == nil then
		isNewPoem = true;
		
		poemId = PoemDB.NextId or "1";
		PoemDB.NextId = tostring(tonumber(poemId) + 1);
		
		self.EditPoemId = poemId;
	else
		ClearArray(WordWrapInfo[poemId]);
	end
	
	poemToSave.Id = poemId;
	PoemDB.Poems[poemId] = poemToSave;
	if poemId == self.SelectedPoemId then
		SaveLines[poemId] = nil;
	end
	
	self:PopulateTagsDropdown(true);

	SavePoems(false);
	
	if isNewPoem then
		self:ClearLines();
		self:LoadAllPoems(true);
	else
		local filter = self.TagsDropdown.SelectedValue;
		local poemIndex = self.PoemsControl:GetIndexForId(poemId);

		local refreshPoems = false;
		if filter ~= nil and filter ~= "" then
			if IsInList(poemToSave.Tags, filter) and poemIndex == nil then
				refreshPoems = true;
			elseif not IsInList(poemToSave.Tags, filter) and poemIndex ~= nil then
				refreshPoems = true;
				if self.SelectedPoemId == poemId then
					self:ClearLines();
				end
			end
		end
		
		if refreshPoems then
			self:LoadAllPoems(true);
		else
			if poemIndex ~= nil then
				self.PoemsControl:UpdateLine(poemIndex, poemToSave.Title, nil);
			end
			if self.SelectedPoemId == poemId then
				self:LoadPoem(poemId);
			end
		end
	end
	
	local popoutWindow = self.Popups[poemId];
	if popoutWindow ~= nil and popoutWindow:IsVisible() then
		popoutWindow:RefreshPoem(false);
	end
end

function PoeticalWindow:DeletePoem(poemId)
	if poemId ~= nil then
		PoemDB.Poems[poemId] = nil;
		SaveLines[poemId] = nil;
		
		self:PopulateTagsDropdown(true);

		SavePoems(false);
		
		self:ClearLines();
		self:LoadAllPoems(false);
	end
end

function PoeticalWindow:ClearLines()
	self.LinesControl:Clear();
	self.DefaultPrefixLabel:SetVisible(false);
end

function PoeticalWindow:ProcessTags()
	local tags = {};
	
	local i = 1;
	
	for _, poem in pairs(PoemDB.Poems) 
	do 
		if poem.Tags ~= nil then
			for _, tag in pairs(poem.Tags) 
			do 
				if not IsInList(tags, tag) then
					table.insert(tags, tag);
					i = i + 1;
				end
			end
		end
	end

	table.sort(tags, function(tagA, tagB)
		return tagA < tagB
	end)
	
	PoemDB.Tags = {};
	for index, tag in ipairs(tags) do
		PoemDB.Tags[tostring(index)] = tag;
	end
end

function PoeticalWindow:PopulateTagsDropdown(processTags)
	if processTags then
		self:ProcessTags();
	end
	
	self.TagsDropdown:ClearItems();
	self.TagsDropdown:AddItem(Strings["no_filter"], "");

	local selectedIndex = 1;
	local selectedTag = self.TagsDropdown.SelectedValue;

	if PoemDB.Tags ~= nil then
		local index = 1;
		local done = false;
		while not done do
			local tag = PoemDB.Tags[index] or PoemDB.Tags[tostring(index)];
			if tag ~= nil then
				self.TagsDropdown:AddItem(tag);
				
				if selectedTag ~= "" and string.ToLower(tag) == string.ToLower(selectedTag) then
					selectedIndex = index + 1;
					self.TagsDropdown.SelectedValue = selectedTag;
				end
			
				index = index + 1;
			else
				done = true;
			end
		end
	end
	
	self.TagsDropdown:SetSelectedIndex(selectedIndex);
end

function PopulatePoeticalListBox(poemId, linesControl, defaultPrefixLabel)
	local poem = nil;
	local bookmarks = {};

	linesControl:Clear();
	
	defaultPrefixLabel:SetVisible(false);

	if poemId ~= nil then
		poem = PoemDB.Poems[poemId];
		
		local defaultPrefix = "";
		local multiPrefix = false;
		
		if poem.UseLocalDefaultCommand then
			defaultPrefix = poem.DefaultCommand;
		elseif Settings.UseLocalDefaultCommand ~= "" then
			defaultPrefix = Settings.DefaultCommand;
		end
		
		if defaultPrefix == nil then
			defaultPrefix = "";
		end
		
		local defaultPrefixList = defaultPrefix:Split(Settings.MulitPrefix);
		local defaultPrefixCount = #defaultPrefixList;
		
		local lineIndex = 1;
		local lineIndexForSplit = 1;
		local done = false;
		
		local wordWrapInfo = WordWrapInfo[poemId];
		if wordWrapInfo == nil then
			WordWrapInfo[poemId] = { };
			wordWrapInfo = WordWrapInfo[poemId];
		end
		
		while not done do
			local text = poem.Lines[tostring(lineIndex)] or poem.Lines[lineIndex];
			if text ~= nil then
				local allAliases = {};
				local allText = {};
				local isSelectable = true;
				local colour = nil;
				local bookmarkName = nil;
				
				local maxCol = 255;
				local red = nil;
				local green = nil;
				local blue = nil;
				
				local r, g, b, t = string.match(text, "^{{(%d+),(%d+),(%d+)}}(.*)");
				if r ~= nil then
					red = tonumber(r) / maxCol;
					green = tonumber(g) / maxCol;
					blue = tonumber(b) / maxCol;
				else
					r, g, b, t = string.match(text, "^{{(%x%x)(%x%x)(%x%x)}}(.*)");
					if r ~= nil then
						red = tonumber(r, 16) / maxCol;
						green = tonumber(g, 16) / maxCol;
						blue = tonumber(b, 16) / maxCol;
					end
				end
				
				allAliases[1] = text;
				allText[1] = text;
				
				if IsInRange(red, 0, maxCol) and IsInRange(green, 0, maxCol) and IsInRange(blue, 0, maxCol) then
					colour = Turbine.UI.Color(1.0, red, green, blue);
					allText[1] = t:Trim();
					allAliases[1] = allText[1];
				else
					bookmarkName, t = string.match(text, BookmarkPatten .. "(.*)");
					if bookmarkName ~= nil then
						allText[1] = t:Trim();
						allAliases[1] = allText[1];
					end
				end
				
				if allText[1] == "" then
					if poem.UseLocalBlankLineCommand then
						allText[1] = poem.BlankLineCommand or "";
						allAliases[1] = allText[1];
					elseif poem.SkipBlankLineCommand then
						isSelectable = false;
					elseif Settings.UseLocalBlankLineCommand then
						allText[1] = Settings.BlankLineCommand or "";
						allAliases[1] = allText[1];
					else
						isSelectable = false;
					end
				elseif Settings.CommentPrefix ~= nil and string.len(Settings.CommentPrefix) > 0 and Settings.CommentPrefix == string.sub(allText[1], 1, string.len(Settings.CommentPrefix)) then
					isSelectable = false;
				elseif string.sub(allText[1], 1, 1) ~= "/" then
					if defaultPrefixCount > 0 then
						allAliases[1] = defaultPrefixList[1] .. " " .. allText[1];
						if Settings.DefaultCommandForEachLine == nil or Settings.DefaultCommandForEachLine or defaultPrefixCount > 1 then
							for i = 2, defaultPrefixCount do
								allAliases[i] = defaultPrefixList[i] .. " " .. allText[1];
								allText[i] = allAliases[i];
							end
							allText[1] = allAliases[1];
						end					
					end
				end
				
				if bookmarkName ~= nil then
					local displayLineIndex = linesControl:GetLineCount() + 1;
					table.insert(bookmarks, { name = bookmarkName, displayLine = displayLineIndex, actualLine = lineIndex });
				end

				for i = 1, #allText do
					local lineAlias = allAliases[i];
					local lineText = allText[i];
					
					local lineLength = wordWrapInfo[lineIndexForSplit];
					if lineLength == nil then
						lineLength = poeticalWindow.WordWrapper:GetTextLength(lineText);
						wordWrapInfo[lineIndexForSplit] = lineLength;
					end
					linesControl:AddLine(lineText, nil, isSelectable, lineAlias, colour, lineLength, i < #allText, lineIndex, i > 1);
					
					lineIndexForSplit = lineIndexForSplit + 1;
				end
					
				lineIndex = lineIndex + 1;
			else
				done = true;
			end
		end

		if (Settings.DefaultCommandForEachLine ~= nil and not Settings.DefaultCommandForEachLine) and #defaultPrefixList == 1 then
			defaultPrefixLabel:SetText(Strings["default_cmd_with_separator"] .. " " .. defaultPrefixList[1]);
			defaultPrefixLabel:SetVisible(true);
		end
		
		linesControl:SetBookmarkInfo(bookmarks, poemId);
	end
	
	return poem;
end

function ClearWordWrapInfo()
	for key, value in pairs (WordWrapInfo) do
		ClearArray(value);
		WordWrapInfo[key] = nil;
	end
end

function ClearArray(array)
	if array ~= nil then
		for index = 1, #array do
			array[index] = nil;
		end
	end
end

function SavePoems(backup)
	if backup then
		PoeticalSave(Turbine.DataScope.Account, "PoeticalPoemsBackup", PoemDB);
	else
		PoeticalSave(Turbine.DataScope.Account, "PoeticalPoems", PoemDB);
	end
end

function DeleteBookmark(poemId, poemLine, bookmarkIndex)
	if poeticalWindow.EditorWindow:IsVisible() and poeticalWindow.EditPoemId == poemId then
		Turbine.Shell.WriteLine("<rgb=#FF0000>" .. Strings["delete_bookmark_warning"] .. "</rgb>");
	else
		local poem = PoemDB.Poems[poemId];
		local line = poem.Lines[tostring(poemLine)];
		
		local newLine = line:gsub(BookmarkPatten, "");
		poem.Lines[tostring(poemLine)] = newLine;
		
		poeticalWindow.LinesControl:RemoveBookmarkIndex(bookmarkIndex);
		
		local popoutWindow = poeticalWindow.Popups[poemId];
		if popoutWindow ~= nil and popoutWindow:IsVisible() then
			popoutWindow.LinesControl:RemoveBookmarkIndex(bookmarkIndex);
		end
		SavePoems(false);
	end
end

function AddBookmark(poemId, poemLine, displayLine, bookmarkName)
	if poeticalWindow.EditorWindow:IsVisible() and poeticalWindow.EditPoemId == poemId then
		Turbine.Shell.WriteLine("<rgb=#FF0000>" .. Strings["add_bookmark_warning"] .. "</rgb>");
	else
		local poem = PoemDB.Poems[poemId];
		local line = poem.Lines[tostring(poemLine)];
		
		local newLine = line:gsub(BookmarkPatten, "");
		poem.Lines[tostring(poemLine)] = BookmarkPrefix .. bookmarkName .. BookmarkSuffix .. poem.Lines[tostring(poemLine)];
		
		local bookmark = { name = bookmarkName, displayLine = displayLine, actualLine = poemLine }
		
		poeticalWindow.LinesControl:AddBookmarkIndex(bookmark);
		
		local popoutWindow = poeticalWindow.Popups[poemId];
		if popoutWindow ~= nil and popoutWindow:IsVisible() then
			popoutWindow.LinesControl:AddBookmarkIndex(bookmark);
		end
		SavePoems(false);
	end
end