import "Simbo.Poetical.PoeticalListBox";
import "Simbo.Poetical.PoeticalLine";

PopoutWindow = class(WindowLotro);

local Constants =
{
	MinimumSize =
	{
		Width = 350,
		Height = 350
	},
	ListBoxes = 
	{	
		Padding = { Left = 15, Top = 45, Right = 15, Bottom = 80}
	},
	Shortcut = 
	{
		Padding = { Bottom = 20 }
	}
}

function PopoutWindow:Constructor()
	WindowLotro.Constructor(self);
	
	self.SaveLine = nil;
	self.PoemId = nil;
	
	self:SetPosition(tonumber(Settings.PoeticalWindow.Position.Left) or 150, tonumber(Settings.PoeticalWindow.Position.Top) or 150);
	self:SetSize(tonumber(Settings.PoeticalWindow.Size.Width) or Constants.MinimumSize.Width, tonumber(Settings.PoeticalWindow.Size.Height) or Constants.MinimumSize.Height);
	self:SetText(Strings["poetical"]);
	self:SetOpacity(GetOpacityFromSettings(Settings.PoeticalWindow));
	self:SetResizer(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
	
	self.Quickslot = ReadButton(self);
	self.Quickslot.ButtonPressed = function(sender, args)
		self.LinesControl:SelectNextLine(self.SaveLine);
	end
	
	self.LinesControl = PoeticalListBox(self, true, nil, self.Quickslot, Settings.WordWrap, true);
	
	self.DefaultPrefixLabel = CreateLabel(self);
	
	self:LayoutControls();
		
	self.LinesControl.SelectedIndexChanged = function(sender, args)
		self.SaveLine = args.SelectedIndex;
	end
end

function PopoutWindow:LayoutControls()
	self.LinesControl:AdjustSizeAndPosition(Constants.ListBoxes.Padding.Left, Constants.ListBoxes.Padding.Top, self:GetWidth() - Constants.ListBoxes.Padding.Left - Constants.ListBoxes.Padding.Right, self:GetHeight() - Constants.ListBoxes.Padding.Top - Constants.ListBoxes.Padding.Bottom);
	self.DefaultPrefixLabel:SetPosition(Constants.ListBoxes.Padding.Left, self.LinesControl:GetTop() + self.LinesControl:GetHeight());
	self.DefaultPrefixLabel:SetWidth(self:GetWidth() - Constants.ListBoxes.Padding.Left - Constants.ListBoxes.Padding.Right);

	self.Quickslot:SetPosition((self:GetWidth() - self.Quickslot:GetWidth()) / 2, self:GetHeight() - Constants.Shortcut.Padding.Bottom - self.Quickslot:GetHeight());
end

function PopoutWindow:Destructor()
	self.LinesControl:Clear();
end

function PopoutWindow:ShowWindow(poemId)
	if not self:IsVisible() and self.PoemId == nil then
		self:SetPosition(poeticalWindow:GetLeft() + 45, poeticalWindow:GetTop() + 45);
		self:SetSize(poeticalWindow:GetWidth(), poeticalWindow:GetHeight());
		self:SetResizerPosition();
	end
	
	self.PoemId = poemId;
	self:RefreshPoem(true);

	if self.SaveLine ~= nil then
		self.LinesControl:SelectLine(self.SaveLine, true);
	end
	
	CheckWindowPosition(self);
	
	self:LayoutControls();
	self:SetVisible(true);
	self:Activate();
end

function PopoutWindow:RefreshPoem(redisplayLine)
	local poem = PoemDB.Poems[self.PoemId];

	PopulatePoeticalListBox(self.PoemId, self.LinesControl, self.DefaultPrefixLabel);
	self:SetText(poem.Title);
	
	if redisplayLine then
		if self.SaveLine ~= nil then
			self.LinesControl:SelectLine(self.SaveLine, true);
		end
	else
		self.SaveLine = nil;
	end
end
