RadioButton = class();

local Constants =
{
	Size =
	{
		ButtonWidth = 16,
		LabelWidth = 20,
		Height = 16
	},
	Spacing = 4,
	Icons =
	{
		Selected = "Simbo/Poetical/Icons/Radio_Selected.tga",
		Unselected = "Simbo/Poetical/Icons/Radio_Unselected.tga"
	}
}

local Groups = {};

local shortcutImages = {};

function RadioButton:Constructor(parent)
	self.SelectedState = false;
	self.GroupName = nil;
	self.Parent = parent;
	
	self.Container = Turbine.UI.Control();
	self.Container:SetParent(parent);
	self.Container:SetSize(Constants.Size.ButtonWidth + Constants.Spacing + Constants.Size.LabelWidth, Constants.Size.Height);
	
	self.Button = Turbine.UI.Control();
	self.Button:SetParent(self.Container);
	self.Button:SetSize(Constants.Size.ButtonWidth, Constants.Size.Height);
	self.Button:SetPosition(0, 0);
	SetIconImage(shortcutImages, self.Button, "Unselected", Constants.Icons);
	self.Button:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);

	self.Label = Turbine.UI.Label();
	self.Label:SetParent(self.Container);
	self.Label:SetForeColor(Globals.Colours.Label);
	self.Label:SetFont(Globals.Fonts.Label);
	self.Label:SetPosition(Constants.Size.ButtonWidth + Constants.Spacing, 0);
	self.Label:SetSize(Constants.Size.LabelWidth, Constants.Size.Height);
	self.Label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.Label:SetMouseVisible(true);
	self.Label:SetText("");

	self.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			if not self.SelectedState then
				self:SetSelected();
				self.Selected(self, {});
			end
		end
	end
	
	self.Button.MouseClick = function(sender, args)
		self.MouseClick(sender, args);
	end
	
	self.Label.MouseClick = function(sender, args)
		self.MouseClick(sender, args);
	end
	
	self.Selected = function(sender, args) end;
end

function RadioButton:SetText(text)
	self.Label:SetText(text);
end

function RadioButton:SetForeColor(colour)
	self.Label:SetForeColor(colour);
end

function RadioButton:SetFont(font)
	self.Label:SetFont(font);
end

function RadioButton:SetPosition(left, top)
	self.Container:SetPosition(left, top);
end

function RadioButton:SetSize(width, height)
	self.Container:SetSize(width, height);
	self.Button:SetTop((height - Constants.Size.Height) / 2);
	self.Label:SetSize(self.Container:GetWidth() - Constants.Size.ButtonWidth - Constants.Spacing, self.Container:GetHeight());
end

function RadioButton:GetLeft()
	return self.Container:GetLeft();
end

function RadioButton:GetTop()
	return self.Container:GetTop();
end

function RadioButton:GetWidth()
	return self.Container:GetWidth();
end

function RadioButton:GetHeight()
	return self.Container:GetHeight();
end

function RadioButton:SetGroup(groupName)
	self.GroupName = tostring(self.Parent) .. ": " .. groupName;
	
	if Groups[self.GroupName] == nil then
		Groups[self.GroupName] = {};
	end
	table.insert(Groups[self.GroupName], self);
end

function RadioButton:SetSelected()
	if not self.SelectedState then
		if self.GroupName ~= nil and Groups[self.GroupName] ~= nil then
			for _, button in ipairs(Groups[self.GroupName]) do
				button:Unselect();
			end
		end
		self.SelectedState = true;
		SetIconImage(shortcutImages, self.Button, "Selected", Constants.Icons);
	end
end

function RadioButton:Unselect()
	self.SelectedState = false;
	SetIconImage(shortcutImages, self.Button, "Unselected", Constants.Icons);
end

function RadioButton:IsSelected()
	return self.SelectedState;
end
