ReadButton = class();

local Constants =
{
	Size = 
	{ 
		Width = 32, 
		Height = 32 
	}
}

local shortcutImages = {};

function ReadButton:Constructor(parent)
	self.UpdateState = 0;
	self.DelayUntil = 0;
	self.UpdateCount = 0;
	self.NoDelay = false;
	self.UpdateControl = Turbine.UI.Control();

	self.Shortcut = Turbine.UI.Lotro.Shortcut();
	self.Shortcut:SetType(Turbine.UI.Lotro.ShortcutType.Alias);
	self.Shortcut:SetData("/");
	
	self.Quickslot = Turbine.UI.Lotro.Quickslot();
	self.Quickslot:SetParent(parent);
	self.Quickslot:SetSize(Constants.Size.Width - 2, Constants.Size.Height - 2);
	self.Quickslot:GetAllowDrop(false);
	self.Quickslot:SetShortcut(self.Shortcut);
	self.Quickslot:SetVisible(false);
	
	-- Fix for dragging short-cut
	self.Quickslot.DragDrop = function(sender, args)
		self.Quickslot:SetShortcut(self.Shortcut); -- restore shortcut
	end
	
	self.QuickslotIcon = Turbine.UI.Control();
	self.QuickslotIcon:SetParent(parent);
	self.QuickslotIcon:SetSize(Constants.Size.Width, Constants.Size.Height);
	self.QuickslotIcon:SetZOrder(self.Quickslot:GetZOrder() + 1);
	self.QuickslotIcon:SetMouseVisible(false);
	self.QuickslotIcon:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	SetIconImage(shortcutImages, self.QuickslotIcon, "MainShortcut");
	self.QuickslotIcon.EnabledChanged = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			SetIconImage(shortcutImages, self.QuickslotIcon, "MainShortcut");
		else
			SetIconImage(shortcutImages, self.QuickslotIcon, "MainShortcut_Inactive");
		end
	end
	self.QuickslotIcon:SetEnabled(false);
	
	self.Quickslot.MouseEnter = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			SetIconImage(shortcutImages, self.QuickslotIcon, "MainShortcut_Hover");
		end
	end
	self.Quickslot.MouseLeave = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			SetIconImage(shortcutImages, self.QuickslotIcon, "MainShortcut");
		end
	end
	self.Quickslot.MouseClick = function(sender, args)
		self.UpdateState = 1;
		self.UpdateControl:SetWantsUpdates(true);
	end
	
	self.UpdateControl.Update = function(sender, args)
		local delay = tonumber(Settings.Delay) or 0;
		local localTime = nil;
		
		if self.NoDelay then
			delay = 0;
		end
		
		if self.UpdateState == 1 then
			self.ButtonPressed(self, {});
			self.UpdateCount = 0;
			if self.QuickslotIcon:IsEnabled() and delay > 0 then
				self.UpdateControl:SetWantsUpdates(true);
				self.UpdateState = 2;
				
				localTime = Turbine.Engine.GetGameTime();
				self.DelayUntil = localTime + delay;
				self:SetEnabled(false, false, self.NoDelay);
			else
				self:CancelUpdate();
			end
		end
		
		if self.UpdateControl:GetWantsUpdates() then
			if localTime == nil then
				localTime = Turbine.Engine.GetGameTime();
			end
			
			self.UpdateCount = self.UpdateCount + 1;
			
			if localTime >= self.DelayUntil and self.UpdateCount > 50 then
				self:SetEnabled(true, nil, self.NoDelay);
				self.UpdateCount = 0;
			end
		end
	end
	
	self.ButtonPressed = function(sender, args) end;
end

function ReadButton:SetPosition(left, top)
	self.QuickslotIcon:SetPosition(left, top);
	self.Quickslot:SetPosition(self.QuickslotIcon:GetLeft() + 1, self.QuickslotIcon:GetTop() + 1);
end

function ReadButton:GetWidth()
	return Constants.Size.Width;
end

function ReadButton:GetHeight()
	return Constants.Size.Height;
end

function ReadButton:SetShortcut(shortcutAlias)
	self.Shortcut:SetData(shortcutAlias);
	self.Quickslot:SetShortcut(self.Shortcut);
end

function ReadButton:SetEnabled(enabled, cancelUpdate, noDelay)
	self.Quickslot:SetVisible(enabled);
	self.QuickslotIcon:SetEnabled(enabled);
	
	self.NoDelay = noDelay or false;
	
	if cancelUpdate == nil or cancelUpdate then
		self:CancelUpdate();
	end
end

function ReadButton:CancelUpdate()
	self.UpdateControl:SetWantsUpdates(false);
	self.UpdateState = 0;
	self.DelayUntil = 0;
end