SettingsWindow = class(WindowLotro);

local Constants =
{
	MinimumSize =
	{
		Width = 520,
		Height = 440
	},
	SettingsBoxes = 
	{	
		Padding = { Left = 15, Top = 55, Right = 15, Bottom = 85},
		Spacing = 8,
		LabelSize = UIOptions["wide_settings_label"] and 200 or 160,
		CheckBoxSize = 200,
		RadioButtonSize = UIOptions["wide_settings_radio_buttons"] and 160 or 120,
		DropdownSize = { Width = UIOptions["wide_settings_dropdown"] and 145 or 100, Height = 24, DropdownHeight = 125 }
	},
	ScrollBars =
	{
		Size = { Width = 250, Height = 10 } 
	},
	Buttons =
	{
		Padding = { Right = 15, Bottom = 20 },
		Spacing = 10
	},
}

function SettingsWindow:Constructor()
	WindowLotro.Constructor(self);
	
	self:SetText(Strings["settings"]);
	self:SetPosition(tonumber(Settings.SettingsWindow.Position.Left) or 200, tonumber(Settings.SettingsWindow.Position.Top) or 200);
	self:SetSize(tonumber(Settings.SettingsWindow.Size.Width) or Constants.MinimumSize.Width, tonumber(Settings.SettingsWindow.Size.Height) or Constants.MinimumSize.Height);
	if self:GetWidth() < Constants.MinimumSize.Width then
		self:SetWidth(Constants.MinimumSize.Width);
	end
	if self:GetHeight() < Constants.MinimumSize.Height then
		self:SetHeight(Constants.MinimumSize.Height);
	end
	
	self:SetOpacity(GetOpacityFromSettings(Settings.SettingsWindow));
	
	local top = Constants.SettingsBoxes.Padding.Top;
	local nextRow = Globals.Sizes.Label.Height + Constants.SettingsBoxes.Spacing;
	local nextJoinedRow = Globals.Sizes.Label.Height / 2 + Constants.SettingsBoxes.Spacing;
	
	local defaultCommandLabel = CreateLabel(self);
	defaultCommandLabel:SetPosition(Constants.SettingsBoxes.Padding.Left, top);
	defaultCommandLabel:SetWidth(Constants.SettingsBoxes.LabelSize);
	defaultCommandLabel:SetText(Strings["default_cmd"]);
	
	self.DefaultCommandBox = CreateWideTextBox(self);
	self.DefaultCommandBox:SetPosition(defaultCommandLabel:GetLeft() + defaultCommandLabel:GetWidth(), top);
		
	top = top + nextJoinedRow;
	
	self.DefaultCommandForEachLineCheckBox = CreateCheckBox(self, Strings["show_each_line"], defaultCommandLabel:GetLeft() + defaultCommandLabel:GetWidth(), top, Constants.SettingsBoxes.CheckBoxSize, Globals.Sizes.Label.Height);
		
	top = top + nextRow;
	
	local blankLineCommandLabel = CreateLabel(self);
	blankLineCommandLabel:SetPosition(Constants.SettingsBoxes.Padding.Left, top);
	blankLineCommandLabel:SetWidth(Constants.SettingsBoxes.LabelSize);
	blankLineCommandLabel:SetText(Strings["blankline_action"]);
	
	self.BlankLineCommandLocalRadioButton = CreateRadioButton(self, Strings["use_command"], "BlankLine", blankLineCommandLabel:GetLeft() + blankLineCommandLabel:GetWidth(), top, Constants.SettingsBoxes.RadioButtonSize, Globals.Sizes.Label.Height);
	
	self.BlankLineCommandBox = CreateTextBox(self);
	self.BlankLineCommandBox:SetPosition(self.BlankLineCommandLocalRadioButton:GetLeft() + self.BlankLineCommandLocalRadioButton:GetWidth(), top);

	top = top + nextJoinedRow;
	
	self.BlankLineCommandSkipRadioButton = CreateRadioButton(self, Strings["skip_line"], "BlankLine", blankLineCommandLabel:GetLeft() + blankLineCommandLabel:GetWidth(), top, Constants.SettingsBoxes.RadioButtonSize, Globals.Sizes.Label.Height);
	
	top = top + nextRow;
	
	local commentPrefixLabel = CreateLabel(self);
	commentPrefixLabel:SetPosition(Constants.SettingsBoxes.Padding.Left, top);
	commentPrefixLabel:SetWidth(Constants.SettingsBoxes.LabelSize);
	commentPrefixLabel:SetText(Strings["comment_prefix"]);
	
	self.CommentPrefixBox = CreateTextBox(self);
	self.CommentPrefixBox:SetPosition(commentPrefixLabel:GetLeft() + commentPrefixLabel:GetWidth(), top);
		
	top = top + nextRow;
	
	local delayLabel = CreateLabel(self);
	delayLabel:SetPosition(Constants.SettingsBoxes.Padding.Left, top);
	delayLabel:SetWidth(Constants.SettingsBoxes.LabelSize);
	delayLabel:SetText(Strings["delay"]);
	
	self.DelayDropdown = CreateDropdown(self, delayLabel:GetLeft() + delayLabel:GetWidth(), top, Constants.SettingsBoxes.DropdownSize.Width, Constants.SettingsBoxes.DropdownSize.Height, Constants.SettingsBoxes.DropdownSize.DropdownHeight);
	self.DelayDropdown:AddItem(Strings["no_delay"], "0");
	for i = 1, 5 do
		self.DelayDropdown:AddItem(tostring(i), tostring(i));
	end
	
	top = top + nextRow;
	
	local wordWrapLabel = CreateLabel(self);
	wordWrapLabel:SetPosition(Constants.SettingsBoxes.Padding.Left, top);
	wordWrapLabel:SetWidth(Constants.SettingsBoxes.LabelSize);
	if UIOptions["tall_word_wrap_label"] then
		wordWrapLabel:SetTop(top - wordWrapLabel:GetHeight() / 2);
		wordWrapLabel:SetHeight(wordWrapLabel:GetHeight() * 2);
	end	
	wordWrapLabel:SetText(Strings["word_wrap"]);
	
	self.WordWrapCheckBox = CreateCheckBox(self, "", wordWrapLabel:GetLeft() + wordWrapLabel:GetWidth(), top, Constants.SettingsBoxes.CheckBoxSize, Globals.Sizes.Label.Height);
		
	top = top + nextRow;
	
	local rememberPositionLabel = CreateLabel(self);
	rememberPositionLabel:SetPosition(Constants.SettingsBoxes.Padding.Left, top);
	rememberPositionLabel:SetWidth(Constants.SettingsBoxes.LabelSize);
	rememberPositionLabel:SetText(Strings["remember_position"]);
	
	self.RememberPositionCheckBox = CreateCheckBox(self, "", rememberPositionLabel:GetLeft() + rememberPositionLabel:GetWidth(), top, Constants.SettingsBoxes.CheckBoxSize, Globals.Sizes.Label.Height);

	top = top + nextRow;
	
	local showActionBarLabel = CreateLabel(self);
	showActionBarLabel:SetPosition(Constants.SettingsBoxes.Padding.Left, top);
	showActionBarLabel:SetWidth(Constants.SettingsBoxes.LabelSize);
	showActionBarLabel:SetText(Strings["display_action_bar"]);
	
	self.ShowActionBarCheckBox = CreateCheckBox(self, "", showActionBarLabel:GetLeft() + showActionBarLabel:GetWidth(), top, Constants.SettingsBoxes.CheckBoxSize, Globals.Sizes.Label.Height);
	
	top = top + nextRow;
	
	local scrollObjects = self:CreateScrollBar(Strings["windows_opacity"], top, 50);
	self.WindowsOpacityScrollBar = scrollObjects.ScrollBar;
	self.WindowsOpacityIndicator = scrollObjects.Indicator;
	self.WindowsOpacityScrollBar.ValueChanged = function(sender,args)
		local newValue = sender:GetValue() / 100;
		self.WindowsOpacityIndicator:SetText(FormatNumber(newValue));
		self:SetOpacity(newValue);
		poeticalWindow:SetOpacity(newValue);
		poeticalWindow:SetPopoutOpacity(newValue);
		poeticalWindow.EditorWindow:SetOpacity(newValue);
	end	
	
	top = top + nextRow;

	scrollObjects = self:CreateScrollBar(Strings["toggle_button_opacity"], top, 0);
	self.ToggleWindowOpacityScrollBar = scrollObjects.ScrollBar;
	self.ToggleWindowOpacityIndicator = scrollObjects.Indicator;
	self.ToggleWindowOpacityScrollBar.ValueChanged = function(sender,args)
		local newValue = sender:GetValue() / 100;
		self.ToggleWindowOpacityIndicator:SetText(FormatNumber(newValue));
		toggleWindow:SetOpacity(newValue);
	end	
	
	self.SaveButton = CreateButton(self);
	self.SaveButton:SetText(Strings["save"]);
	self.SaveButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			local changed = self:SaveSettings();
			args = { Changed = changed };
			self.Saved(sender, args);
			self:Close();
		end
	end	
	
	self.CancelButton = CreateButton(self);
	self.CancelButton:SetText(Strings["cancel"]);
	self.CancelButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			self:Close();
		end
	end	
	
	self.Saved = function() end;
	
	self.VisibleChanged = function(sender, args)
		if not self:IsVisible() then
			self:SetOpacity(GetOpacityFromSettings(Settings.SettingsWindow));
			poeticalWindow:SetOpacity(GetOpacityFromSettings(Settings.PoeticalWindow));
			poeticalWindow:SetPopoutOpacity(GetOpacityFromSettings(Settings.PoeticalWindow));
			poeticalWindow.EditorWindow:SetOpacity(GetOpacityFromSettings(Settings.EditorWindow));
			toggleWindow:SetOpacity(GetOpacityFromSettings(Settings.ToggleWindow));
		else
			CheckWindowPosition(self);
		end
	end
	
	self:LayoutControls();
	self:SetResizer(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
end

function SettingsWindow:CreateScrollBar(labelText, top, minimum)
	local opacityLabel = CreateLabel(self);
	opacityLabel:SetPosition(Constants.SettingsBoxes.Padding.Left, top);
	opacityLabel:SetWidth(Constants.SettingsBoxes.LabelSize);
	opacityLabel:SetText(labelText);
	
	local scrollBarWidth = Constants.ScrollBars.Size.Width * (100 - minimum) / 100;
	
	local opacityScroll = Turbine.UI.Lotro.ScrollBar();
	opacityScroll:SetParent(self);
	opacityScroll:SetOrientation(Turbine.UI.Orientation.Horizontal);
	opacityScroll:SetPosition(Constants.SettingsBoxes.Padding.Left + Constants.SettingsBoxes.LabelSize, top + (opacityLabel:GetHeight() - Constants.ScrollBars.Size.Height) / 2);
	opacityScroll:SetSize(scrollBarWidth, Constants.ScrollBars.Size.Height);
	opacityScroll:SetValue(100);
	opacityScroll:SetMaximum(100);
	opacityScroll:SetMinimum(minimum);
	opacityScroll:SetSmallChange(1);
	opacityScroll:SetLargeChange(5);
	
	local opacityInd = CreateLabel(self);
	opacityInd:SetFont(Globals.Fonts.SmallLabel);
	opacityInd:SetPosition(Constants.SettingsBoxes.Padding.Left + Constants.SettingsBoxes.LabelSize + scrollBarWidth, top);
	opacityInd:SetWidth(30);
	opacityInd:SetText(FormatNumber(1));
	
	return { Label = opacityLabel, ScrollBar = opacityScroll, Indicator = opacityInd }
end

function SettingsWindow:ShowWindow()
	self.DefaultCommandBox:SetText(Settings.DefaultCommand);
	self.DefaultCommandForEachLineCheckBox:SetChecked(Settings.DefaultCommandForEachLine);
	
	if Settings.UseLocalBlankLineCommand then
		self.BlankLineCommandLocalRadioButton:SetSelected();
	else
		self.BlankLineCommandSkipRadioButton:SetSelected();	
	end
	self.BlankLineCommandBox:SetText(Settings.BlankLineCommand);
	
	self.CommentPrefixBox:SetText(Settings.CommentPrefix);
	self.DelayDropdown:SetSelectedValue(tostring(tonumber(Settings.Delay) or 0));
	self.WordWrapCheckBox:SetChecked(Settings.WordWrap);
	self.RememberPositionCheckBox:SetChecked(Settings.RememberPosition);
	self.ShowActionBarCheckBox:SetChecked(Settings.ActionBarVisible);
	
	self.WindowsOpacityScrollBar:SetValue(GetOpacityFromSettings(Settings.PoeticalWindow) * 100);
	self.ToggleWindowOpacityScrollBar:SetValue(GetOpacityFromSettings(Settings.ToggleWindow) * 100);
	
	self:SetVisible(true);
	self:Activate();
end

function SettingsWindow:SaveSettings()
	local changed = false;
	
	local textSetting = self.DefaultCommandBox:GetText():Trim();
	if Settings.DefaultCommand ~= textSetting then
		Settings.DefaultCommand = textSetting;
		changed = true;
	end
	
	local checkBoxSetting = self.DefaultCommandForEachLineCheckBox:IsChecked();
	if Settings.DefaultCommandForEachLine == nil or Settings.DefaultCommandForEachLine ~= checkBoxSetting then
		Settings.DefaultCommandForEachLine = checkBoxSetting;
		changed = true;
	end
	
	checkBoxSetting = self.BlankLineCommandLocalRadioButton:IsSelected();
	if Settings.UseLocalBlankLineCommand ~= checkBoxSetting then
		Settings.UseLocalBlankLineCommand = checkBoxSetting;
		changed = true;
	end
	
	textSetting = self.BlankLineCommandBox:GetText():Trim();
	if Settings.BlankLineCommand ~= textSetting then
		Settings.BlankLineCommand = textSetting;
		changed = true;
	end
	
	textSetting = self.CommentPrefixBox:GetText():Trim();
	if Settings.CommentPrefix ~= textSetting then
		Settings.CommentPrefix = textSetting;
		changed = true;
	end
	
	textSetting = self.DelayDropdown:GetSelectedValue();
	if Settings.Delay ~= textSetting then
		Settings.Delay = textSetting;
		changed = true;
	end
	
	checkBoxSetting = self.WordWrapCheckBox:IsChecked();
	if Settings.WordWrap == nil or Settings.WordWrap ~= checkBoxSetting then
		Settings.WordWrap = checkBoxSetting;
		changed = true;
	end
	
	checkBoxSetting = self.RememberPositionCheckBox:IsChecked();
	if Settings.RememberPosition == nil or Settings.RememberPosition ~= checkBoxSetting then
		Settings.RememberPosition = checkBoxSetting;
		changed = true;
	end
	
	checkBoxSetting = self.ShowActionBarCheckBox:IsChecked();
	if Settings.ActionBarVisible == nil or Settings.ActionBarVisible ~= checkBoxSetting then
		Settings.ActionBarVisible = checkBoxSetting;
		changed = true;
	end
	
	local scrollSetting = self.WindowsOpacityScrollBar:GetValue();
	if GetOpacityFromSettings(Settings.PoeticalWindow) * 100 ~= scrollSetting then
		Settings.PoeticalWindow.Opacity = tostring(scrollSetting);
		Settings.SettingsWindow.Opacity = tostring(scrollSetting);
		Settings.EditorWindow.Opacity = tostring(scrollSetting);
		changed = true;
	end
	
	scrollSetting = self.ToggleWindowOpacityScrollBar:GetValue();
	if GetOpacityFromSettings(Settings.ToggleWindow) * 100 ~= scrollSetting then
		Settings.ToggleWindow.Opacity = tostring(scrollSetting);
		changed = true;
	end
	
	return changed;
end

function SettingsWindow:LayoutControls()
	local width = self:GetWidth() - Constants.SettingsBoxes.Padding.Left - Constants.SettingsBoxes.Padding.Right;
	local boxWidth = width - Constants.SettingsBoxes.LabelSize;
	
	self.BlankLineCommandBox:SetWidth(boxWidth - Constants.SettingsBoxes.RadioButtonSize);
	
	self.SaveButton:SetPosition(self:GetWidth() - Constants.Buttons.Padding.Right - Globals.Sizes.Button.Width, self:GetHeight() - Constants.Buttons.Padding.Bottom - Globals.Sizes.Button.Height);
	self.CancelButton:SetPosition(self.SaveButton:GetLeft() - Constants.Buttons.Spacing - Globals.Sizes.Button.Width, self.SaveButton:GetTop());
end
